#!/usr/bin/env python

# Author: Dr. Konstantin Selyunin
# License: MIT

import argparse
import json
import sys

from argparse import RawTextHelpFormatter


def um7_autodetect(file_name=None):
    """
    :param file_name: by default ID of sensor will be assigned
    :return: id file will be created in target directory
    """
    json_config = {}
    if sys.platform.startswith('win32'):
        from serial.tools import list_ports
        device_list = list_ports.comports()
        for device in device_list:
            # go through device list and stop on first FTDI device
            if device.manufacturer == "FTDI":
                # found something, add keys
                # device.device is the port name (COM4 for example)
                json_config['ID_VENDOR'] = device.manufacturer
                json_config['ID_SERIAL_SHORT'] = device.serial_number
                # don't know what the model should look like so were making it "pid:vid"
                json_config['ID_MODEL'] = '{0:04x}:{1:04x}'.format(device.vid, device.pid)
                if file_name is None:
                    file_name = f"um7_{json_config['ID_SERIAL_SHORT']}.json"
                with open(file_name, 'w') as fp:
                    json.dump(json_config, fp, indent=2)
                return True, json_config
        # none of the devices was FTDI
        return False, {}
    else:
        import pyudev
        context = pyudev.Context()
        udev_keys = ['ID_VENDOR', 'ID_SERIAL_SHORT', 'ID_MODEL']
        for device in context.list_devices(subsystem='tty'):
            if device.get('ID_VENDOR') == 'FTDI':
                for key in udev_keys:
                    json_config[key] = device.get(key)
                if file_name is None:
                    file_name = f"um7_{json_config['ID_SERIAL_SHORT']}.json"
                with open(file_name, 'w') as fp:
                    json.dump(json_config, fp, indent=2)
                return True, json_config
        else:
            return False, {}


description = \
"""
Store UM7 udev parameters on a Linux-based system.
UM7 uses FTDI chip, and has no specific
descriptor information to differentiate itself 
from other FTDI devices (e.g. stepper motors or other
devices implementing USB2Serial connection). 

In order to differentiate these devices, we 
have this script, which searches for available 
devices and stores information in a .json file.
\n
Follow the procedure below:
-------------------------------
\n
1. Disconnect all the USB2Serial-related devices;
2. Connect UM7 sensor;
3. Launch the script (with additional -f flag, which specifies file name);
4. The file `um7_[serial_id].json` containing device 
specific information appears on success.

This file will be used by UM7 drivers
"""

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description=description, formatter_class=RawTextHelpFormatter)
    parser.add_argument('-f', '--file', type=str, help='')
    args = parser.parse_args(sys.argv[1:])
    ok, config = um7_autodetect(args.file)
    print(config)
    if not ok:
        print("FAILED to store ID of UM7! Check whether the sensor is connected!")



# Author: Dr. Konstantin Selyunin
# Created: 7 April 2020
# Modified: 12 June 2020
# Version: v0.5
# License: MIT

import os.path

from rsl_comm_py.rsl_generator import RslGenerator
from textwrap import indent
from datetime import datetime


if __name__ == '__main__':
    script_folder = os.path.dirname(__file__)
    svd_file = os.path.join(script_folder, os.pardir, 'rsl_xml_svd/shearwater.svd')
    rsl_svd_generator = RslGenerator(svd_file=svd_file)
    shearwater_main_registers = indent(rsl_svd_generator.generate_props_for_main_register_map(), ' ' * 4)
    shearwater_hidden_registers = indent(rsl_svd_generator.generate_props_for_hidden_registers(), ' ' * 4)

    today = datetime.now().strftime('%Y.%m.%d')
    params_dict = {
        'generated_code_for_main_register_map': shearwater_main_registers,
        'generated_code_for_hidden_register_map': shearwater_hidden_registers,
        'today': today
    }
    shearwater_template = os.path.join(script_folder, os.pardir, 'um7py/templates/shearwater_template.jinja2')
    gen_code = RslGenerator.render_template_to_str(shearwater_template, params_dict)
    with open('shearwater_registers.py', 'w') as fd:
        fd.write(gen_code)

    reg_addr_enum_template = os.path.join(script_folder, os.pardir, 'um7py/templates/python_reg_access.jinja2')
    param_dict = {'version': 'v0.2',
                  'date': today,
                  'cregs': rsl_svd_generator.cregs,
                  'dregs': rsl_svd_generator.dregs,
                  'commands': rsl_svd_generator.commands,
                  'module': 'shearwater_serial',
                  'classname': 'ShearWaterSerial',
                  'svd': 'shearwater',
                  'object': 'shearwater'}
    gen_code = RslGenerator.render_template_to_str(reg_addr_enum_template, param_dict)
    with open('shearwater_py_accessor.py', 'w') as fd:
        fd.write(gen_code)

    reg_map_template = os.path.join(script_folder, os.pardir, 'um7py/templates/register_map.h.jinja2')
    param_dict = {'version': 'v0.2',
                  'date': today,
                  'regs': rsl_svd_generator.regs,
                  'define_guard': 'RSL_SHEARWATER_REGISTER_MAP_H'}
    gen_code = RslGenerator.render_template_to_str(reg_map_template, param_dict)
    with open('shearwater.h', 'w') as fd:
        fd.write(gen_code)

    reg_addr_enum_template = os.path.join(script_folder, os.pardir, 'um7py/templates/register_enum.h.jinja2')
    param_dict = {'version': 'v0.3',
                  'date': today,
                  'cregs': rsl_svd_generator.cregs,
                  'dregs': rsl_svd_generator.dregs,
                  'commands': rsl_svd_generator.commands,
                  'define_guard': 'RSL_SHEARWATER_REGISTER_ENUM_H'}
    gen_code = RslGenerator.render_template_to_str(reg_addr_enum_template, param_dict)
    with open('shearwater_enum.h', 'w') as fd:
        fd.write(gen_code)

    #  HIDDEN_REGISTERS
    hidden_reg_map_template = os.path.join(script_folder, os.pardir, 'um7py/templates/register_map.h.jinja2')
    reg_map_out = 'shearwater_hidden.h'
    param_dict = {'version': 'v0.2',
                  'date': today,
                  'regs': rsl_svd_generator.hidden_regs,
                  'define_guard': 'RSL_SHEARWATER_HIDDEN_REGISTER_MAP_H'}
    gen_code = RslGenerator.render_template_to_str(hidden_reg_map_template, param_dict)
    with open('shearwater_hidden.h', 'w') as fd:
        fd.write(gen_code)

    reg_addr_enum_template = os.path.join(script_folder, os.pardir, 'um7py/templates/register_hidden_enum.h.jinja2')
    param_dict = {'version': 'v0.2',
                  'date': today,
                  'regs': rsl_svd_generator.hidden_regs,
                  'define_guard':  'RSL_SHEARWATER_HIDDEN_REGISTER_ENUM_MAP_H'}
    gen_code = RslGenerator.render_template_to_str(reg_addr_enum_template, param_dict)
    with open('shearwater_hidden_enum.h', 'w') as fd:
        fd.write(gen_code)

    reg_hidden_enum_template = os.path.join(script_folder, os.pardir, 'um7py/templates/python_hidden_reg_access.jinja2')
    param_dict = {'version': 'v0.2',
                  'date': today,
                  'regs': rsl_svd_generator.hidden_regs}
    gen_code = RslGenerator.render_template_to_str(reg_hidden_enum_template, param_dict)
    with open('shearwater_hidden_py_accessor.py', 'w') as fd:
        fd.write(gen_code)

    reg_python_config_template = os.path.join(script_folder, os.pardir,
                                              'um7py/templates/python_shearwater_config.jinja2')
    param_dict = {'hidden_regs': rsl_svd_generator.hidden_regs,
                  'config_regs': rsl_svd_generator.cregs,
                  'data_regs': rsl_svd_generator.dregs,
                  'command_regs': rsl_svd_generator.commands}
    gen_code = RslGenerator.render_template_to_str(reg_python_config_template, param_dict)
    with open('ShearwaterConfiguration.py', 'w') as fd:
        fd.write(gen_code)



#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 30 May 2020

import logging
import os.path
import sys

from um7py.um7_serial import UM7Serial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log', mode='w'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNP8.json")
    um7 = UM7Serial(device=device_file)
    print("This script lists all available commands for UM7, running these commands might erase your settings.\n"
          "Executing these commands in order does not have any meaning, all commands are listed for reference only.\n"
          "Change the code in this fine to run all the commands you need.")
    know_what_will_happen = False
    if not know_what_will_happen:
        print("NO commands will be executed, Get firmware revision and exit.\n"
              "Everything is fine, all the settings stay unchanged.\n"
              "To execute the commands change the `know_what_will_happen` to `True` and select commands you need.")
        print(f"get_fw_revision               : {um7.get_fw_revision}")
    else:
        print(f"\\n========== COMMAND REGISTERS ===================================")
        print(f"get_fw_revision               : {um7.get_fw_revision}")
        print(f"flash_commit                  : {um7.flash_commit}")
        print(f"reset_to_factory              : {um7.reset_to_factory}")
        print(f"zero_gyros                    : {um7.zero_gyros}")
        print(f"set_home_position             : {um7.set_home_position}")
        print(f"set_mag_reference             : {um7.set_mag_reference}")
        print(f"calibrate_accelerometers      : {um7.calibrate_accelerometers}")
        print(f"reset_ekf                     : {um7.reset_ekf}")




#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# Date: 12 June 2020
# Version: v0.3
# License: MIT

import logging
import os.path
import struct

from time import sleep
from typing import List, Union, Tuple

from rsl_comm_py.um7_serial import RslException


class SpiCommunication:
    def __init__(self, *args, **kwargs):
        super().__init__(**kwargs)

    def connect(self, *args, **kwargs):
        pass

    def xfer(self, msg):
        raise NotImplemented("This method should be implemented in child classes!")

    def spi_xfer(self, msg):
        # self.ssn_pin.state = False
        # sleep(0.05)
        response = self.xfer(msg)
        logging.info(f'msg: {msg}\t\tresponse: {response}')
        # sleep(0.01)
        # self.ssn_pin.state = True
        return response

    def read_register(self, reg_addr: int, **kw) -> Tuple[bool, bytes]:
        msg = [0x00, reg_addr] + [0x00] * 4
        response = self.spi_xfer(msg)
        return True, bytes(response[2:])

    def write_register(self, reg_addr: int, reg_value: Union[int, bytes, float], **kw):
        if type(reg_value) == float:
            reg_value = struct.pack('>f', reg_value)
        elif type(reg_value) == bytes:
            reg_value = list(reg_value)
        elif type(reg_value) == int:
            reg_value = int.to_bytes(reg_value, length=4, byteorder='big', signed=False)
        msg = [0x01, reg_addr] + list(reg_value)
        self.spi_xfer(msg)
        return True

    def read_consecutive_registers(self, reg_addr: int, num_registers: int):
        msg = [0x00, reg_addr] + [0x00] * 4 * num_registers
        response = self.spi_xfer(msg)
        return True, bytes(response[2:])


class RslSpiLinuxPort(SpiCommunication):
    def __init__(self, *args, **kwargs):
        import spidev
        super().__init__(args, kwargs)
        self.bus = kwargs.get('bus') if kwargs.get('bus') is not None else 0
        self.device = kwargs.get('device') if kwargs.get('device') is not None else 0
        self.spi_device_path = f'/dev/spidev{self.bus}.{self.device}'
        if not os.path.exists(self.spi_device_path):
            raise RslException(f'SPI device not found: {self.spi_device_path}')
        self.spi = spidev.SpiDev()
        self.connect()

    def connect(self, *args, **kwargs):
        self.spi.open(self.bus, self.device)
        self.spi.max_speed_hz = 500000

    def xfer(self, bytes_to_send: List[int]) -> List[int]:
        tx_bytes = bytes_to_send[:]
        self.spi.xfer(tx_bytes)
        return tx_bytes


class RslSpiUsbIss(SpiCommunication):
    def __init__(self, **kwargs):
        from usb_iss import UsbIss
        super().__init__(**kwargs)
        self.iss = UsbIss()
        self.port = kwargs.get('port') if kwargs.get('port') is not None else '/dev/ttyACM0'
        import platform
        if platform.system().lower() == 'linux':
            if not os.path.exists(self.port):
                raise RslException(f'SPI port not found: {self.port}: USB-ISS needs to connect to ACM'
                                   f' port for SPI communication. The port does not exist, specify `port` argument!')
        self.connect()

    def connect(self, *args, **kwargs):
        self.iss.open(self.port)
        self.iss.setup_spi(clock_khz=200)

    def xfer(self, bytes_to_send: List[int]) -> List[int]:
        recv_bytes = self.iss.spi.transfer(bytes_to_send)
        return recv_bytes


if __name__ == '__main__':
    pass



import logging
import os.path
import pytest
import sys

from um7py.shearwater_serial import ShearWaterSerial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNHD.json")
    shearwater = ShearWaterSerial(device=device_file)

    print(f"\n========== HIDDEN REGISTERS ===================================")
    print(f"hidden_gyro_1_variance        : {shearwater.hidden_gyro_1_variance}")
    print(f"hidden_gyro_2_variance        : {shearwater.hidden_gyro_2_variance}")
    print(f"hidden_accel_1_variance       : {shearwater.hidden_accel_1_variance}")
    print(f"hidden_mag_1_variance         : {shearwater.hidden_mag_1_variance}")
    print(f"hidden_mag_2_variance         : {shearwater.hidden_mag_2_variance}")
    print(f"hidden_gps_course_variance    : {shearwater.hidden_gps_course_variance}")
    print(f"hidden_gps_position_variance  : {shearwater.hidden_gps_position_variance}")
    print(f"hidden_gps_velocity_variance  : {shearwater.hidden_gps_velocity_variance}")
    print(f"hidden_static_press_variance  : {shearwater.hidden_static_press_variance}")
    print(f"hidden_diff_press_variance    : {shearwater.hidden_diff_press_variance}")
    print(f"hidden_q_uvw                  : {shearwater.hidden_q_uvw}")
    print(f"hidden_q_quaternion           : {shearwater.hidden_q_quaternion}")
    print(f"hidden_q_gps_position         : {shearwater.hidden_q_gps_position}")
    print(f"hidden_q_bias                 : {shearwater.hidden_q_bias}")
    print(f"hidden_q_euler_angles         : {shearwater.hidden_q_euler_angles}")
    print(f"hidden_low_vg_accel_noise_factor: {shearwater.hidden_low_vg_accel_noise_factor}")
    print(f"hidden_lpf_tau_groundspeed    : {shearwater.hidden_lpf_tau_groundspeed}")
    print(f"hidden_lpf_tau_gyro_1         : {shearwater.hidden_lpf_tau_gyro_1}")
    print(f"hidden_lpf_tau_gyro_2         : {shearwater.hidden_lpf_tau_gyro_2}")
    print(f"hidden_lpf_tau_accel_1        : {shearwater.hidden_lpf_tau_accel_1}")
    print(f"hidden_lpf_tau_mag_1          : {shearwater.hidden_lpf_tau_mag_1}")
    print(f"hidden_lpf_tau_mag_2          : {shearwater.hidden_lpf_tau_mag_2}")
    print(f"hidden_c_gyro_1_bias_x_pow_0  : {shearwater.hidden_c_gyro_1_bias_x_pow_0}")
    print(f"hidden_c_gyro_1_bias_x_pow_1  : {shearwater.hidden_c_gyro_1_bias_x_pow_1}")
    print(f"hidden_c_gyro_1_bias_x_pow_2  : {shearwater.hidden_c_gyro_1_bias_x_pow_2}")
    print(f"hidden_c_gyro_1_bias_x_pow_3  : {shearwater.hidden_c_gyro_1_bias_x_pow_3}")
    print(f"hidden_c_gyro_1_bias_y_pow_0  : {shearwater.hidden_c_gyro_1_bias_y_pow_0}")
    print(f"hidden_c_gyro_1_bias_y_pow_1  : {shearwater.hidden_c_gyro_1_bias_y_pow_1}")
    print(f"hidden_c_gyro_1_bias_y_pow_2  : {shearwater.hidden_c_gyro_1_bias_y_pow_2}")
    print(f"hidden_c_gyro_1_bias_y_pow_3  : {shearwater.hidden_c_gyro_1_bias_y_pow_3}")
    print(f"hidden_c_gyro_1_bias_z_pow_0  : {shearwater.hidden_c_gyro_1_bias_z_pow_0}")
    print(f"hidden_c_gyro_1_bias_z_pow_1  : {shearwater.hidden_c_gyro_1_bias_z_pow_1}")
    print(f"hidden_c_gyro_1_bias_z_pow_2  : {shearwater.hidden_c_gyro_1_bias_z_pow_2}")
    print(f"hidden_c_gyro_1_bias_z_pow_3  : {shearwater.hidden_c_gyro_1_bias_z_pow_3}")
    print(f"hidden_c_gyro_1_scale_x_pow_0 : {shearwater.hidden_c_gyro_1_scale_x_pow_0}")
    print(f"hidden_c_gyro_1_scale_x_pow_1 : {shearwater.hidden_c_gyro_1_scale_x_pow_1}")
    print(f"hidden_c_gyro_1_scale_x_pow_2 : {shearwater.hidden_c_gyro_1_scale_x_pow_2}")
    print(f"hidden_c_gyro_1_scale_x_pow_3 : {shearwater.hidden_c_gyro_1_scale_x_pow_3}")
    print(f"hidden_c_gyro_1_scale_y_pow_0 : {shearwater.hidden_c_gyro_1_scale_y_pow_0}")
    print(f"hidden_c_gyro_1_scale_y_pow_1 : {shearwater.hidden_c_gyro_1_scale_y_pow_1}")
    print(f"hidden_c_gyro_1_scale_y_pow_2 : {shearwater.hidden_c_gyro_1_scale_y_pow_2}")
    print(f"hidden_c_gyro_1_scale_y_pow_3 : {shearwater.hidden_c_gyro_1_scale_y_pow_3}")
    print(f"hidden_c_gyro_1_scale_z_pow_0 : {shearwater.hidden_c_gyro_1_scale_z_pow_0}")
    print(f"hidden_c_gyro_1_scale_z_pow_1 : {shearwater.hidden_c_gyro_1_scale_z_pow_1}")
    print(f"hidden_c_gyro_1_scale_z_pow_2 : {shearwater.hidden_c_gyro_1_scale_z_pow_2}")
    print(f"hidden_c_gyro_1_scale_z_pow_3 : {shearwater.hidden_c_gyro_1_scale_z_pow_3}")
    print(f"hidden_gyro_1_alignment1_1    : {shearwater.hidden_gyro_1_alignment1_1}")
    print(f"hidden_gyro_1_alignment1_2    : {shearwater.hidden_gyro_1_alignment1_2}")
    print(f"hidden_gyro_1_alignment1_3    : {shearwater.hidden_gyro_1_alignment1_3}")
    print(f"hidden_gyro_1_alignment2_1    : {shearwater.hidden_gyro_1_alignment2_1}")
    print(f"hidden_gyro_1_alignment2_2    : {shearwater.hidden_gyro_1_alignment2_2}")
    print(f"hidden_gyro_1_alignment2_3    : {shearwater.hidden_gyro_1_alignment2_3}")
    print(f"hidden_gyro_1_alignment3_1    : {shearwater.hidden_gyro_1_alignment3_1}")
    print(f"hidden_gyro_1_alignment3_2    : {shearwater.hidden_gyro_1_alignment3_2}")
    print(f"hidden_gyro_1_alignment3_3    : {shearwater.hidden_gyro_1_alignment3_3}")
    print(f"hidden_c_gyro_2_bias_x_pow_0  : {shearwater.hidden_c_gyro_2_bias_x_pow_0}")
    print(f"hidden_c_gyro_2_bias_x_pow_1  : {shearwater.hidden_c_gyro_2_bias_x_pow_1}")
    print(f"hidden_c_gyro_2_bias_x_pow_2  : {shearwater.hidden_c_gyro_2_bias_x_pow_2}")
    print(f"hidden_c_gyro_2_bias_x_pow_3  : {shearwater.hidden_c_gyro_2_bias_x_pow_3}")
    print(f"hidden_c_gyro_2_bias_y_pow_0  : {shearwater.hidden_c_gyro_2_bias_y_pow_0}")
    print(f"hidden_c_gyro_2_bias_y_pow_1  : {shearwater.hidden_c_gyro_2_bias_y_pow_1}")
    print(f"hidden_c_gyro_2_bias_y_pow_2  : {shearwater.hidden_c_gyro_2_bias_y_pow_2}")
    print(f"hidden_c_gyro_2_bias_y_pow_3  : {shearwater.hidden_c_gyro_2_bias_y_pow_3}")
    print(f"hidden_c_gyro_2_bias_z_pow_0  : {shearwater.hidden_c_gyro_2_bias_z_pow_0}")
    print(f"hidden_c_gyro_2_bias_z_pow_1  : {shearwater.hidden_c_gyro_2_bias_z_pow_1}")
    print(f"hidden_c_gyro_2_bias_z_pow_2  : {shearwater.hidden_c_gyro_2_bias_z_pow_2}")
    print(f"hidden_c_gyro_2_bias_z_pow_3  : {shearwater.hidden_c_gyro_2_bias_z_pow_3}")
    print(f"hidden_c_gyro_2_scale_x_pow_0 : {shearwater.hidden_c_gyro_2_scale_x_pow_0}")
    print(f"hidden_c_gyro_2_scale_x_pow_1 : {shearwater.hidden_c_gyro_2_scale_x_pow_1}")
    print(f"hidden_c_gyro_2_scale_x_pow_2 : {shearwater.hidden_c_gyro_2_scale_x_pow_2}")
    print(f"hidden_c_gyro_2_scale_x_pow_3 : {shearwater.hidden_c_gyro_2_scale_x_pow_3}")
    print(f"hidden_c_gyro_2_scale_y_pow_0 : {shearwater.hidden_c_gyro_2_scale_y_pow_0}")
    print(f"hidden_c_gyro_2_scale_y_pow_1 : {shearwater.hidden_c_gyro_2_scale_y_pow_1}")
    print(f"hidden_c_gyro_2_scale_y_pow_2 : {shearwater.hidden_c_gyro_2_scale_y_pow_2}")
    print(f"hidden_c_gyro_2_scale_y_pow_3 : {shearwater.hidden_c_gyro_2_scale_y_pow_3}")
    print(f"hidden_c_gyro_2_scale_z_pow_0 : {shearwater.hidden_c_gyro_2_scale_z_pow_0}")
    print(f"hidden_c_gyro_2_scale_z_pow_1 : {shearwater.hidden_c_gyro_2_scale_z_pow_1}")
    print(f"hidden_c_gyro_2_scale_z_pow_2 : {shearwater.hidden_c_gyro_2_scale_z_pow_2}")
    print(f"hidden_c_gyro_2_scale_z_pow_3 : {shearwater.hidden_c_gyro_2_scale_z_pow_3}")
    print(f"hidden_gyro_2_alignment1_1    : {shearwater.hidden_gyro_2_alignment1_1}")
    print(f"hidden_gyro_2_alignment1_2    : {shearwater.hidden_gyro_2_alignment1_2}")
    print(f"hidden_gyro_2_alignment1_3    : {shearwater.hidden_gyro_2_alignment1_3}")
    print(f"hidden_gyro_2_alignment2_1    : {shearwater.hidden_gyro_2_alignment2_1}")
    print(f"hidden_gyro_2_alignment2_2    : {shearwater.hidden_gyro_2_alignment2_2}")
    print(f"hidden_gyro_2_alignment2_3    : {shearwater.hidden_gyro_2_alignment2_3}")
    print(f"hidden_gyro_2_alignment3_1    : {shearwater.hidden_gyro_2_alignment3_1}")
    print(f"hidden_gyro_2_alignment3_2    : {shearwater.hidden_gyro_2_alignment3_2}")
    print(f"hidden_gyro_2_alignment3_3    : {shearwater.hidden_gyro_2_alignment3_3}")
    print(f"hidden_c_accel_1_bias_x_pow_0 : {shearwater.hidden_c_accel_1_bias_x_pow_0}")
    print(f"hidden_c_accel_1_bias_x_pow_1 : {shearwater.hidden_c_accel_1_bias_x_pow_1}")
    print(f"hidden_c_accel_1_bias_x_pow_2 : {shearwater.hidden_c_accel_1_bias_x_pow_2}")
    print(f"hidden_c_accel_1_bias_x_pow_3 : {shearwater.hidden_c_accel_1_bias_x_pow_3}")
    print(f"hidden_c_accel_1_bias_y_pow_0 : {shearwater.hidden_c_accel_1_bias_y_pow_0}")
    print(f"hidden_c_accel_1_bias_y_pow_1 : {shearwater.hidden_c_accel_1_bias_y_pow_1}")
    print(f"hidden_c_accel_1_bias_y_pow_2 : {shearwater.hidden_c_accel_1_bias_y_pow_2}")
    print(f"hidden_c_accel_1_bias_y_pow_3 : {shearwater.hidden_c_accel_1_bias_y_pow_3}")
    print(f"hidden_c_accel_1_bias_z_pow_0 : {shearwater.hidden_c_accel_1_bias_z_pow_0}")
    print(f"hidden_c_accel_1_bias_z_pow_1 : {shearwater.hidden_c_accel_1_bias_z_pow_1}")
    print(f"hidden_c_accel_1_bias_z_pow_2 : {shearwater.hidden_c_accel_1_bias_z_pow_2}")
    print(f"hidden_c_accel_1_bias_z_pow_3 : {shearwater.hidden_c_accel_1_bias_z_pow_3}")
    print(f"hidden_c_accel_1_scale_x_pow_0: {shearwater.hidden_c_accel_1_scale_x_pow_0}")
    print(f"hidden_c_accel_1_scale_x_pow_1: {shearwater.hidden_c_accel_1_scale_x_pow_1}")
    print(f"hidden_c_accel_1_scale_x_pow_2: {shearwater.hidden_c_accel_1_scale_x_pow_2}")
    print(f"hidden_c_accel_1_scale_x_pow_3: {shearwater.hidden_c_accel_1_scale_x_pow_3}")
    print(f"hidden_c_accel_1_scale_y_pow_0: {shearwater.hidden_c_accel_1_scale_y_pow_0}")
    print(f"hidden_c_accel_1_scale_y_pow_1: {shearwater.hidden_c_accel_1_scale_y_pow_1}")
    print(f"hidden_c_accel_1_scale_y_pow_2: {shearwater.hidden_c_accel_1_scale_y_pow_2}")
    print(f"hidden_c_accel_1_scale_y_pow_3: {shearwater.hidden_c_accel_1_scale_y_pow_3}")
    print(f"hidden_c_accel_1_scale_z_pow_0: {shearwater.hidden_c_accel_1_scale_z_pow_0}")
    print(f"hidden_c_accel_1_scale_z_pow_1: {shearwater.hidden_c_accel_1_scale_z_pow_1}")
    print(f"hidden_c_accel_1_scale_z_pow_2: {shearwater.hidden_c_accel_1_scale_z_pow_2}")
    print(f"hidden_c_accel_1_scale_z_pow_3: {shearwater.hidden_c_accel_1_scale_z_pow_3}")
    print(f"hidden_accel_1_alignment1_1   : {shearwater.hidden_accel_1_alignment1_1}")
    print(f"hidden_accel_1_alignment1_2   : {shearwater.hidden_accel_1_alignment1_2}")
    print(f"hidden_accel_1_alignment1_3   : {shearwater.hidden_accel_1_alignment1_3}")
    print(f"hidden_accel_1_alignment2_1   : {shearwater.hidden_accel_1_alignment2_1}")
    print(f"hidden_accel_1_alignment2_2   : {shearwater.hidden_accel_1_alignment2_2}")
    print(f"hidden_accel_1_alignment2_3   : {shearwater.hidden_accel_1_alignment2_3}")
    print(f"hidden_accel_1_alignment3_1   : {shearwater.hidden_accel_1_alignment3_1}")
    print(f"hidden_accel_1_alignment3_2   : {shearwater.hidden_accel_1_alignment3_2}")
    print(f"hidden_accel_1_alignment3_3   : {shearwater.hidden_accel_1_alignment3_3}")
    print(f"hidden_c_mag_1_bias_x_pow_0   : {shearwater.hidden_c_mag_1_bias_x_pow_0}")
    print(f"hidden_c_mag_1_bias_x_pow_1   : {shearwater.hidden_c_mag_1_bias_x_pow_1}")
    print(f"hidden_c_mag_1_bias_x_pow_2   : {shearwater.hidden_c_mag_1_bias_x_pow_2}")
    print(f"hidden_c_mag_1_bias_x_pow_3   : {shearwater.hidden_c_mag_1_bias_x_pow_3}")
    print(f"hidden_c_mag_1_bias_y_pow_0   : {shearwater.hidden_c_mag_1_bias_y_pow_0}")
    print(f"hidden_c_mag_1_bias_y_pow_1   : {shearwater.hidden_c_mag_1_bias_y_pow_1}")
    print(f"hidden_c_mag_1_bias_y_pow_2   : {shearwater.hidden_c_mag_1_bias_y_pow_2}")
    print(f"hidden_c_mag_1_bias_y_pow_3   : {shearwater.hidden_c_mag_1_bias_y_pow_3}")
    print(f"hidden_c_mag_1_bias_z_pow_0   : {shearwater.hidden_c_mag_1_bias_z_pow_0}")
    print(f"hidden_c_mag_1_bias_z_pow_1   : {shearwater.hidden_c_mag_1_bias_z_pow_1}")
    print(f"hidden_c_mag_1_bias_z_pow_2   : {shearwater.hidden_c_mag_1_bias_z_pow_2}")
    print(f"hidden_c_mag_1_bias_z_pow_3   : {shearwater.hidden_c_mag_1_bias_z_pow_3}")
    print(f"hidden_c_mag_1_scale_x_pow_0  : {shearwater.hidden_c_mag_1_scale_x_pow_0}")
    print(f"hidden_c_mag_1_scale_x_pow_1  : {shearwater.hidden_c_mag_1_scale_x_pow_1}")
    print(f"hidden_c_mag_1_scale_x_pow_2  : {shearwater.hidden_c_mag_1_scale_x_pow_2}")
    print(f"hidden_c_mag_1_scale_x_pow_3  : {shearwater.hidden_c_mag_1_scale_x_pow_3}")
    print(f"hidden_c_mag_1_scale_y_pow_0  : {shearwater.hidden_c_mag_1_scale_y_pow_0}")
    print(f"hidden_c_mag_1_scale_y_pow_1  : {shearwater.hidden_c_mag_1_scale_y_pow_1}")
    print(f"hidden_c_mag_1_scale_y_pow_2  : {shearwater.hidden_c_mag_1_scale_y_pow_2}")
    print(f"hidden_c_mag_1_scale_y_pow_3  : {shearwater.hidden_c_mag_1_scale_y_pow_3}")
    print(f"hidden_c_mag_1_scale_z_pow_0  : {shearwater.hidden_c_mag_1_scale_z_pow_0}")
    print(f"hidden_c_mag_1_scale_z_pow_1  : {shearwater.hidden_c_mag_1_scale_z_pow_1}")
    print(f"hidden_c_mag_1_scale_z_pow_2  : {shearwater.hidden_c_mag_1_scale_z_pow_2}")
    print(f"hidden_c_mag_1_scale_z_pow_3  : {shearwater.hidden_c_mag_1_scale_z_pow_3}")
    print(f"hidden_mag_1_alignment1_1     : {shearwater.hidden_mag_1_alignment1_1}")
    print(f"hidden_mag_1_alignment1_2     : {shearwater.hidden_mag_1_alignment1_2}")
    print(f"hidden_mag_1_alignment1_3     : {shearwater.hidden_mag_1_alignment1_3}")
    print(f"hidden_mag_1_alignment2_1     : {shearwater.hidden_mag_1_alignment2_1}")
    print(f"hidden_mag_1_alignment2_2     : {shearwater.hidden_mag_1_alignment2_2}")
    print(f"hidden_mag_1_alignment2_3     : {shearwater.hidden_mag_1_alignment2_3}")
    print(f"hidden_mag_1_alignment3_1     : {shearwater.hidden_mag_1_alignment3_1}")
    print(f"hidden_mag_1_alignment3_2     : {shearwater.hidden_mag_1_alignment3_2}")
    print(f"hidden_mag_1_alignment3_3     : {shearwater.hidden_mag_1_alignment3_3}")
    print(f"hidden_mag_1_reference_x      : {shearwater.hidden_mag_1_reference_x}")
    print(f"hidden_mag_1_reference_y      : {shearwater.hidden_mag_1_reference_y}")
    print(f"hidden_mag_1_reference_z      : {shearwater.hidden_mag_1_reference_z}")
    print(f"hidden_c_mag_2_bias_x_pow_0   : {shearwater.hidden_c_mag_2_bias_x_pow_0}")
    print(f"hidden_c_mag_2_bias_x_pow_1   : {shearwater.hidden_c_mag_2_bias_x_pow_1}")
    print(f"hidden_c_mag_2_bias_x_pow_2   : {shearwater.hidden_c_mag_2_bias_x_pow_2}")
    print(f"hidden_c_mag_2_bias_x_pow_3   : {shearwater.hidden_c_mag_2_bias_x_pow_3}")
    print(f"hidden_c_mag_2_bias_y_pow_0   : {shearwater.hidden_c_mag_2_bias_y_pow_0}")
    print(f"hidden_c_mag_2_bias_y_pow_1   : {shearwater.hidden_c_mag_2_bias_y_pow_1}")
    print(f"hidden_c_mag_2_bias_y_pow_2   : {shearwater.hidden_c_mag_2_bias_y_pow_2}")
    print(f"hidden_c_mag_2_bias_y_pow_3   : {shearwater.hidden_c_mag_2_bias_y_pow_3}")
    print(f"hidden_c_mag_2_bias_z_pow_0   : {shearwater.hidden_c_mag_2_bias_z_pow_0}")
    print(f"hidden_c_mag_2_bias_z_pow_1   : {shearwater.hidden_c_mag_2_bias_z_pow_1}")
    print(f"hidden_c_mag_2_bias_z_pow_2   : {shearwater.hidden_c_mag_2_bias_z_pow_2}")
    print(f"hidden_c_mag_2_bias_z_pow_3   : {shearwater.hidden_c_mag_2_bias_z_pow_3}")
    print(f"hidden_c_mag_2_scale_x_pow_0  : {shearwater.hidden_c_mag_2_scale_x_pow_0}")
    print(f"hidden_c_mag_2_scale_x_pow_1  : {shearwater.hidden_c_mag_2_scale_x_pow_1}")
    print(f"hidden_c_mag_2_scale_x_pow_2  : {shearwater.hidden_c_mag_2_scale_x_pow_2}")
    print(f"hidden_c_mag_2_scale_x_pow_3  : {shearwater.hidden_c_mag_2_scale_x_pow_3}")
    print(f"hidden_c_mag_2_scale_y_pow_0  : {shearwater.hidden_c_mag_2_scale_y_pow_0}")
    print(f"hidden_c_mag_2_scale_y_pow_1  : {shearwater.hidden_c_mag_2_scale_y_pow_1}")
    print(f"hidden_c_mag_2_scale_y_pow_2  : {shearwater.hidden_c_mag_2_scale_y_pow_2}")
    print(f"hidden_c_mag_2_scale_y_pow_3  : {shearwater.hidden_c_mag_2_scale_y_pow_3}")
    print(f"hidden_c_mag_2_scale_z_pow_0  : {shearwater.hidden_c_mag_2_scale_z_pow_0}")
    print(f"hidden_c_mag_2_scale_z_pow_1  : {shearwater.hidden_c_mag_2_scale_z_pow_1}")
    print(f"hidden_c_mag_2_scale_z_pow_2  : {shearwater.hidden_c_mag_2_scale_z_pow_2}")
    print(f"hidden_c_mag_2_scale_z_pow_3  : {shearwater.hidden_c_mag_2_scale_z_pow_3}")
    print(f"hidden_mag_2_alignment1_1     : {shearwater.hidden_mag_2_alignment1_1}")
    print(f"hidden_mag_2_alignment1_2     : {shearwater.hidden_mag_2_alignment1_2}")
    print(f"hidden_mag_2_alignment1_3     : {shearwater.hidden_mag_2_alignment1_3}")
    print(f"hidden_mag_2_alignment2_1     : {shearwater.hidden_mag_2_alignment2_1}")
    print(f"hidden_mag_2_alignment2_2     : {shearwater.hidden_mag_2_alignment2_2}")
    print(f"hidden_mag_2_alignment2_3     : {shearwater.hidden_mag_2_alignment2_3}")
    print(f"hidden_mag_2_alignment3_1     : {shearwater.hidden_mag_2_alignment3_1}")
    print(f"hidden_mag_2_alignment3_2     : {shearwater.hidden_mag_2_alignment3_2}")
    print(f"hidden_mag_2_alignment3_3     : {shearwater.hidden_mag_2_alignment3_3}")
    print(f"hidden_mag_2_reference_x      : {shearwater.hidden_mag_2_reference_x}")
    print(f"hidden_mag_2_reference_y      : {shearwater.hidden_mag_2_reference_y}")
    print(f"hidden_mag_2_reference_z      : {shearwater.hidden_mag_2_reference_z}")
    print(f"hidden_gyro_1_conversion      : {shearwater.hidden_gyro_1_conversion}")
    print(f"hidden_gyro_2_conversion      : {shearwater.hidden_gyro_2_conversion}")
    print(f"hidden_accel_1_conversion     : {shearwater.hidden_accel_1_conversion}")
    print(f"hidden_mag_1_conversion       : {shearwater.hidden_mag_1_conversion}")
    print(f"hidden_mag_2_conversion       : {shearwater.hidden_mag_2_conversion}")



#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 31 March 2021

import logging
import os.path
import sys
from time import sleep

from um7py.shearwater_serial import ShearWaterSerial
from um7py.shearwater_spi import ShearWaterSpiUsbIss


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNHD.json")

    shearwater_spi = ShearWaterSpiUsbIss(port='/dev/ttyACM0')
    shearwater_serial = ShearWaterSerial(device=device_file)

    print(f"\n========== DATA REGISTERS ===================================")
    for _ in range(100):
        print(f"dreg_mag_1_raw_x_spi            : {shearwater_spi.dreg_mag_1_raw_x[0].raw_value}")
        print(f"dreg_mag_1_raw_x_serial         : {shearwater_serial.dreg_mag_1_raw_x[0].raw_value}")
        print(f"dreg_mag_1_raw_y_spi            : {shearwater_spi.dreg_mag_1_raw_y[0].raw_value}")
        print(f"dreg_mag_1_raw_y_serial         : {shearwater_serial.dreg_mag_1_raw_y[0].raw_value}")
        print(f"dreg_mag_1_raw_z_spi            : {shearwater_spi.dreg_mag_1_raw_z[0].raw_value}")
        print(f"dreg_mag_1_raw_z_serial         : {shearwater_serial.dreg_mag_1_raw_z[0].raw_value}")
        print(f"dreg_mag_1_raw_time_spi         : {shearwater_spi.dreg_mag_1_raw_time[0].raw_value}")
        print(f"dreg_mag_1_raw_time_serial      : {shearwater_serial.dreg_mag_1_raw_time[0].raw_value}")
        print(f"dreg_mag_2_raw_xy_spi           : {shearwater_spi.dreg_mag_2_raw_xy[0].raw_value}")
        print(f"dreg_mag_2_raw_xy_serial        : {shearwater_serial.dreg_mag_2_raw_xy[0].raw_value}")
        print(f"dreg_mag_2_raw_z_spi            : {shearwater_spi.dreg_mag_2_raw_z[0].raw_value}")
        print(f"dreg_mag_2_raw_z_serial         : {shearwater_serial.dreg_mag_2_raw_z[0].raw_value}")
        print(f"dreg_mag_2_raw_time_spi         : {shearwater_spi.dreg_mag_2_raw_time[0].raw_value}")
        print(f"dreg_mag_2_raw_time_serial      : {shearwater_serial.dreg_mag_2_raw_time[0].raw_value}")
        print(f"dreg_temperature_spi            : {shearwater_spi.dreg_temperature[0].raw_value}")
        print(f"dreg_temperature_serial         : {shearwater_serial.dreg_temperature[0].raw_value}")
        print(f"dreg_temperature_time_spi       : {shearwater_spi.dreg_temperature_time[0].raw_value}")
        print(f"dreg_temperature_time_serial    : {shearwater_serial.dreg_temperature_time[0].raw_value}")
        print(f"dreg_gyro_1_proc_x_spi          : {shearwater_spi.dreg_gyro_1_proc_x[0].raw_value}")
        print(f"dreg_gyro_1_proc_x_serial       : {shearwater_serial.dreg_gyro_1_proc_x[0].raw_value}")
        print(f"dreg_gyro_1_proc_y_spi          : {shearwater_spi.dreg_gyro_1_proc_y[0].raw_value}")
        print(f"dreg_gyro_1_proc_y_serial       : {shearwater_serial.dreg_gyro_1_proc_y[0].raw_value}")
        print(f"dreg_gyro_1_proc_z_spi          : {shearwater_spi.dreg_gyro_1_proc_z[0].raw_value}")
        print(f"dreg_gyro_1_proc_z_serial       : {shearwater_serial.dreg_gyro_1_proc_z[0].raw_value}")
        print(f"dreg_gyro_1_proc_time_spi       : {shearwater_spi.dreg_gyro_1_proc_time[0].raw_value}")
        print(f"dreg_gyro_1_proc_time_serial    : {shearwater_serial.dreg_gyro_1_proc_time[0].raw_value}")
        print(f"dreg_gyro_2_proc_x_spi          : {shearwater_spi.dreg_gyro_2_proc_x[0].raw_value}")
        print(f"dreg_gyro_2_proc_x_serial       : {shearwater_serial.dreg_gyro_2_proc_x[0].raw_value}")
        print(f"dreg_gyro_2_proc_y_spi          : {shearwater_spi.dreg_gyro_2_proc_y[0].raw_value}")
        print(f"dreg_gyro_2_proc_y_serial       : {shearwater_serial.dreg_gyro_2_proc_y[0].raw_value}")
        print(f"dreg_gyro_2_proc_z_spi          : {shearwater_spi.dreg_gyro_2_proc_z[0].raw_value}")
        print(f"dreg_gyro_2_proc_z_serial       : {shearwater_serial.dreg_gyro_2_proc_z[0].raw_value}")
        print(f"dreg_gyro_2_proc_time_spi       : {shearwater_spi.dreg_gyro_2_proc_time[0].raw_value}")
        print(f"dreg_gyro_2_proc_time_serial    : {shearwater_serial.dreg_gyro_2_proc_time[0].raw_value}")
        print(f"dreg_accel_1_proc_x_spi         : {shearwater_spi.dreg_accel_1_proc_x[0].raw_value}")
        print(f"dreg_accel_1_proc_x_serial      : {shearwater_serial.dreg_accel_1_proc_x[0].raw_value}")
        print(f"dreg_accel_1_proc_y_spi         : {shearwater_spi.dreg_accel_1_proc_y[0].raw_value}")
        print(f"dreg_accel_1_proc_y_serial      : {shearwater_serial.dreg_accel_1_proc_y[0].raw_value}")
        print(f"dreg_accel_1_proc_z_spi         : {shearwater_spi.dreg_accel_1_proc_z[0].raw_value}")
        print(f"dreg_accel_1_proc_z_serial      : {shearwater_serial.dreg_accel_1_proc_z[0].raw_value}")
        print(f"dreg_accel_1_proc_time_spi      : {shearwater_spi.dreg_accel_1_proc_time[0].raw_value}")
        print(f"dreg_accel_1_proc_time_serial   : {shearwater_serial.dreg_accel_1_proc_time[0].raw_value}")
        print(f"dreg_mag_1_proc_x_spi           : {shearwater_spi.dreg_mag_1_proc_x[0].raw_value}")
        print(f"dreg_mag_1_proc_x_serial        : {shearwater_serial.dreg_mag_1_proc_x[0].raw_value}")
        print(f"dreg_mag_1_proc_y_spi           : {shearwater_spi.dreg_mag_1_proc_y[0].raw_value}")
        print(f"dreg_mag_1_proc_y_serial        : {shearwater_serial.dreg_mag_1_proc_y[0].raw_value}")
        print(f"dreg_mag_1_proc_z_spi           : {shearwater_spi.dreg_mag_1_proc_z[0].raw_value}")
        print(f"dreg_mag_1_proc_z_serial        : {shearwater_serial.dreg_mag_1_proc_z[0].raw_value}")
        print(f"dreg_mag_1_norm_spi             : {shearwater_spi.dreg_mag_1_norm[0].raw_value}")
        print(f"dreg_mag_1_norm_serial          : {shearwater_serial.dreg_mag_1_norm[0].raw_value}")
        print(f"dreg_mag_1_proc_time_spi        : {shearwater_spi.dreg_mag_1_proc_time[0].raw_value}")
        print(f"dreg_mag_1_proc_time_serial     : {shearwater_serial.dreg_mag_1_proc_time[0].raw_value}")
        print(f"dreg_mag_2_proc_x_spi           : {shearwater_spi.dreg_mag_2_proc_x[0].raw_value}")
        print(f"dreg_mag_2_proc_x_serial        : {shearwater_serial.dreg_mag_2_proc_x[0].raw_value}")
        print(f"dreg_mag_2_proc_y_spi           : {shearwater_spi.dreg_mag_2_proc_y[0].raw_value}")
        print(f"dreg_mag_2_proc_y_serial        : {shearwater_serial.dreg_mag_2_proc_y[0].raw_value}")
        print(f"dreg_mag_2_proc_z_spi           : {shearwater_spi.dreg_mag_2_proc_z[0].raw_value}")
        print(f"dreg_mag_2_proc_z_serial        : {shearwater_serial.dreg_mag_2_proc_z[0].raw_value}")
        print(f"dreg_mag_2_norm_spi             : {shearwater_spi.dreg_mag_2_norm[0].raw_value}")
        print(f"dreg_mag_2_norm_serial          : {shearwater_serial.dreg_mag_2_norm[0].raw_value}")
        print(f"dreg_mag_2_proc_time_spi        : {shearwater_spi.dreg_mag_2_proc_time[0].raw_value}")
        print(f"dreg_mag_2_proc_time_serial     : {shearwater_serial.dreg_mag_2_proc_time[0].raw_value}")
        print(f"dreg_quat_ab_spi                : {shearwater_spi.dreg_quat_ab[0].raw_value}")
        print(f"dreg_quat_ab_serial             : {shearwater_serial.dreg_quat_ab[0].raw_value}")
        print(f"dreg_quat_cd_spi                : {shearwater_spi.dreg_quat_cd[0].raw_value}")
        print(f"dreg_quat_cd_serial             : {shearwater_serial.dreg_quat_cd[0].raw_value}")
        print(f"dreg_quat_time_spi              : {shearwater_spi.dreg_quat_time[0].raw_value}")
        print(f"dreg_quat_time_serial           : {shearwater_serial.dreg_quat_time[0].raw_value}")
        print(f"dreg_euler_phi_theta_spi        : {shearwater_spi.dreg_euler_phi_theta[0].raw_value}")
        print(f"dreg_euler_phi_theta_serial     : {shearwater_serial.dreg_euler_phi_theta[0].raw_value}")
        print(f"dreg_euler_psi_spi              : {shearwater_spi.dreg_euler_psi[0].raw_value}")
        print(f"dreg_euler_psi_serial           : {shearwater_serial.dreg_euler_psi[0].raw_value}")
        print(f"dreg_euler_phi_theta_dot_spi    : {shearwater_spi.dreg_euler_phi_theta_dot[0].raw_value}")
        print(f"dreg_euler_phi_theta_dot_serial : {shearwater_serial.dreg_euler_phi_theta_dot[0].raw_value}")
        print(f"dreg_euler_psi_dot_spi          : {shearwater_spi.dreg_euler_psi_dot[0].raw_value}")
        print(f"dreg_euler_psi_dot_serial       : {shearwater_serial.dreg_euler_psi_dot[0].raw_value}")
        print(f"dreg_euler_time_spi             : {shearwater_spi.dreg_euler_time[0].raw_value}")
        print(f"dreg_euler_time_serial          : {shearwater_serial.dreg_euler_time[0].raw_value}")




#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 3 May 2020

import logging
import os.path
import sys

from um7py.shearwater_serial import ShearWaterSerial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNHD.json")
    shearwater = ShearWaterSerial(device=device_file)

    print(f"\n========== DATA REGISTERS ===================================")
    print(f"dreg_health                   : {shearwater.dreg_health}")
    print(f"dreg_gyro_1_raw_xy            : {shearwater.dreg_gyro_1_raw_xy}")
    print(f"dreg_gyro_1_raw_z             : {shearwater.dreg_gyro_1_raw_z}")
    print(f"dreg_gyro_1_raw_time          : {shearwater.dreg_gyro_1_raw_time}")
    print(f"dreg_gyro_2_raw_xy            : {shearwater.dreg_gyro_2_raw_xy}")
    print(f"dreg_gyro_2_raw_z             : {shearwater.dreg_gyro_2_raw_z}")
    print(f"dreg_gyro_2_raw_time          : {shearwater.dreg_gyro_2_raw_time}")
    print(f"dreg_accel_1_raw_xy           : {shearwater.dreg_accel_1_raw_xy}")
    print(f"dreg_accel_1_raw_z            : {shearwater.dreg_accel_1_raw_z}")
    print(f"dreg_accel_1_raw_time         : {shearwater.dreg_accel_1_raw_time}")
    print(f"dreg_mag_1_raw_x              : {shearwater.dreg_mag_1_raw_x}")
    print(f"dreg_mag_1_raw_y              : {shearwater.dreg_mag_1_raw_y}")
    print(f"dreg_mag_1_raw_z              : {shearwater.dreg_mag_1_raw_z}")
    print(f"dreg_mag_1_raw_time           : {shearwater.dreg_mag_1_raw_time}")
    print(f"dreg_mag_2_raw_xy             : {shearwater.dreg_mag_2_raw_xy}")
    print(f"dreg_mag_2_raw_z              : {shearwater.dreg_mag_2_raw_z}")
    print(f"dreg_mag_2_raw_time           : {shearwater.dreg_mag_2_raw_time}")
    print(f"dreg_temperature              : {shearwater.dreg_temperature}")
    print(f"dreg_temperature_time         : {shearwater.dreg_temperature_time}")
    print(f"dreg_gyro_1_proc_x            : {shearwater.dreg_gyro_1_proc_x}")
    print(f"dreg_gyro_1_proc_y            : {shearwater.dreg_gyro_1_proc_y}")
    print(f"dreg_gyro_1_proc_z            : {shearwater.dreg_gyro_1_proc_z}")
    print(f"dreg_gyro_1_proc_time         : {shearwater.dreg_gyro_1_proc_time}")
    print(f"dreg_gyro_2_proc_x            : {shearwater.dreg_gyro_2_proc_x}")
    print(f"dreg_gyro_2_proc_y            : {shearwater.dreg_gyro_2_proc_y}")
    print(f"dreg_gyro_2_proc_z            : {shearwater.dreg_gyro_2_proc_z}")
    print(f"dreg_gyro_2_proc_time         : {shearwater.dreg_gyro_2_proc_time}")
    print(f"dreg_accel_1_proc_x           : {shearwater.dreg_accel_1_proc_x}")
    print(f"dreg_accel_1_proc_y           : {shearwater.dreg_accel_1_proc_y}")
    print(f"dreg_accel_1_proc_z           : {shearwater.dreg_accel_1_proc_z}")
    print(f"dreg_accel_1_proc_time        : {shearwater.dreg_accel_1_proc_time}")
    print(f"dreg_mag_1_proc_x             : {shearwater.dreg_mag_1_proc_x}")
    print(f"dreg_mag_1_proc_y             : {shearwater.dreg_mag_1_proc_y}")
    print(f"dreg_mag_1_proc_z             : {shearwater.dreg_mag_1_proc_z}")
    print(f"dreg_mag_1_norm               : {shearwater.dreg_mag_1_norm}")
    print(f"dreg_mag_1_proc_time          : {shearwater.dreg_mag_1_proc_time}")
    print(f"dreg_mag_2_proc_x             : {shearwater.dreg_mag_2_proc_x}")
    print(f"dreg_mag_2_proc_y             : {shearwater.dreg_mag_2_proc_y}")
    print(f"dreg_mag_2_proc_z             : {shearwater.dreg_mag_2_proc_z}")
    print(f"dreg_mag_2_norm               : {shearwater.dreg_mag_2_norm}")
    print(f"dreg_mag_2_proc_time          : {shearwater.dreg_mag_2_proc_time}")
    print(f"dreg_quat_ab                  : {shearwater.dreg_quat_ab}")
    print(f"dreg_quat_cd                  : {shearwater.dreg_quat_cd}")
    print(f"dreg_quat_time                : {shearwater.dreg_quat_time}")
    print(f"dreg_euler_phi_theta          : {shearwater.dreg_euler_phi_theta}")
    print(f"dreg_euler_psi                : {shearwater.dreg_euler_psi}")
    print(f"dreg_euler_phi_theta_dot      : {shearwater.dreg_euler_phi_theta_dot}")
    print(f"dreg_euler_psi_dot            : {shearwater.dreg_euler_psi_dot}")
    print(f"dreg_euler_time               : {shearwater.dreg_euler_time}")
    print(f"dreg_position_north           : {shearwater.dreg_position_north}")
    print(f"dreg_position_east            : {shearwater.dreg_position_east}")
    print(f"dreg_position_up              : {shearwater.dreg_position_up}")
    print(f"dreg_position_time            : {shearwater.dreg_position_time}")
    print(f"dreg_velocity_north           : {shearwater.dreg_velocity_north}")
    print(f"dreg_velocity_east            : {shearwater.dreg_velocity_east}")
    print(f"dreg_velocity_up              : {shearwater.dreg_velocity_up}")
    print(f"dreg_velocity_time            : {shearwater.dreg_velocity_time}")
    print(f"dreg_gyro_1_bias_x            : {shearwater.dreg_gyro_1_bias_x}")
    print(f"dreg_gyro_1_bias_y            : {shearwater.dreg_gyro_1_bias_y}")
    print(f"dreg_gyro_1_bias_z            : {shearwater.dreg_gyro_1_bias_z}")
    print(f"dreg_gyro_2_bias_x            : {shearwater.dreg_gyro_2_bias_x}")
    print(f"dreg_gyro_2_bias_y            : {shearwater.dreg_gyro_2_bias_y}")
    print(f"dreg_gyro_2_bias_z            : {shearwater.dreg_gyro_2_bias_z}")




import logging
import os.path
import sys
from um7py.shearwater_serial import ShearWaterSerial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNHD.json")
    shearwater = ShearWaterSerial(device=device_file)
    print(f"\n========== CHANGING MEASUREMENT RANGE EXAMPLE ==========================")
    print(f"Setting measurement range to default values:")
    shearwater.creg_gyro_1_meas_range = 0
    shearwater.creg_gyro_2_meas_range = 0
    shearwater.creg_accel_1_meas_range = 0
    print(f"creg_gyro_1_meas_range        : {shearwater.creg_gyro_1_meas_range}")
    print(f"creg_gyro_2_meas_range        : {shearwater.creg_gyro_2_meas_range}")
    print(f"creg_accel_1_meas_range       : {shearwater.creg_accel_1_meas_range}")
    print(f"Conversion coefficients:")
    print(f"hidden_gyro_1_conversion      : {shearwater.hidden_gyro_1_conversion}")
    print(f"hidden_gyro_2_conversion      : {shearwater.hidden_gyro_2_conversion}")
    print(f"hidden_accel_1_conversion     : {shearwater.hidden_accel_1_conversion}")
    print(f"hidden_mag_1_conversion       : {shearwater.hidden_mag_1_conversion}")
    print(f"hidden_mag_2_conversion       : {shearwater.hidden_mag_2_conversion}")
    print(f"Changing measurement range for all the sensors...")
    shearwater.creg_gyro_1_meas_range = 1
    shearwater.creg_gyro_2_meas_range = 2
    shearwater.creg_accel_1_meas_range = 3
    print(f"Updated config registers:")
    print(f"creg_gyro_1_meas_range        : {shearwater.creg_gyro_1_meas_range}")
    print(f"creg_gyro_2_meas_range        : {shearwater.creg_gyro_2_meas_range}")
    print(f"creg_accel_1_meas_range       : {shearwater.creg_accel_1_meas_range}")
    print(f"Updated conversion coefficients:")
    print(f"hidden_gyro_1_conversion      : {shearwater.hidden_gyro_1_conversion}")
    print(f"hidden_gyro_2_conversion      : {shearwater.hidden_gyro_2_conversion}")
    print(f"hidden_accel_1_conversion     : {shearwater.hidden_accel_1_conversion}")
    print(f"hidden_mag_1_conversion       : {shearwater.hidden_mag_1_conversion}")
    print(f"hidden_mag_2_conversion       : {shearwater.hidden_mag_2_conversion}")




#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 08 August 2020


import logging
import os.path
import sys

from um7py.um7_serial import UM7Serial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d]: %(message)s',
        datefmt='%H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log', mode='w'),
            logging.StreamHandler(sys.stdout),
        ])
    um7 = UM7Serial(port_name='/dev/ttyUSB0')

    print("um7 firmware revision: {}".format(um7.get_fw_revision))
    creg_com_settings, *_ = um7.creg_com_settings
    print("um7 creg_com_settings: {}".format(um7.creg_com_settings))

    print(f"setting new baud rate: 921600 baud")
    # look at the register description -->
    # https://docs.redshiftlabs.com.au/register_map_current.html#creg-com-settings
    # 3 of BAUD_RATE corresponds to the 921600 BAUD
    creg_com_settings.set_field_value(BAUD_RATE=11)
    # we have now set a raw register value, let us write it in the sensor
    print(creg_com_settings)
    um7.creg_com_settings = creg_com_settings.raw_value

    # let us update the baud rate of the communication object:
    from time import sleep
    sleep(2)
    um7.port.close()
    um7.port.baudrate = 921600
    um7.port.open()
    # let us try to read out firmware revision now, with the new rate:
    print("um7 firmware revision: {}".format(um7.get_fw_revision))

    print("Getting broadcast packets with new baud rate: ")
    for packet in um7.recv_broadcast(num_packets=10):
        logging.warning(packet)

    sleep(1)
    # let us restore baud rate back to 115200
    print(f"Setting baud rate back to 115200")
    creg_com_settings.set_field_value(BAUD_RATE=5)
    um7.creg_com_settings = creg_com_settings.raw_value

    um7.port.close()
    um7.port.baudrate = 115200
    um7.port.open()

    sleep(1)

    for packet in um7.recv_broadcast(num_packets=10):
        logging.warning(packet)




#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 28 March 2021
# NOTE: this only works with python3.8+ !
import json
import logging
from pathlib import Path
import sys

from um7py.um7_serial import UM7Serial
from um7py.um7_broadcast_packets import UM7AllProcPacket, UM7AllRawPacket
from multiprocessing import shared_memory, Process, Lock
from time import sleep, time


BUFFER_SIZE = 1000
raw_data_shm = shared_memory.SharedMemory(create=True, size=BUFFER_SIZE)
proc_data_shm = shared_memory.SharedMemory(create=True, size=BUFFER_SIZE)
raw_lock = Lock()
proc_lock = Lock()


def sensor_read_process(raw_shm: shared_memory.SharedMemory, proc_shm: shared_memory.SharedMemory, r_lock: Lock, p_lock: Lock):
    script_dir = Path(__file__).parent
    device_file = script_dir.parent.joinpath("um7_A500CNP8.json")
    assert device_file.exists(), f"Device file with connection info: {device_file} does not exist!"
    um7 = UM7Serial(device=str(device_file))

    for packet in um7.recv_broadcast(flush_buffer_on_start=False):
        packet_bytes = bytes(json.dumps(packet.__dict__), encoding='utf-8')
        assert len(packet_bytes) <= BUFFER_SIZE, f"Packet cannot be serialized, increase `BUFFER` size at least up to {len(packet_bytes)}"
        if isinstance(packet, UM7AllRawPacket):
            r_lock.acquire()
            raw_shm.buf[:] = b' ' * BUFFER_SIZE
            raw_shm.buf[:len(packet_bytes)] = packet_bytes
            r_lock.release()
            # logging.warning(f"[SR][RAW] -> {packet}")
        elif isinstance(packet, UM7AllProcPacket):
            p_lock.acquire()
            proc_shm.buf[:] = b' ' * BUFFER_SIZE
            proc_shm.buf[:len(packet_bytes)] = packet_bytes
            p_lock.release()
            # logging.warning(f"[SR][PROC] -> {packet}")


def main_function():
    start_time = time()

    idx = 0
    while time() - start_time < 60:
        idx += 1
        if idx % 2 == 0:
            # imagine I need to process raw data now
            raw_lock.acquire()
            raw_meas_bytes: bytes = raw_data_shm.buf[:]
            raw_lock.release()
            raw_meas_str = str(raw_meas_bytes, encoding='utf-8')
            raw_meas_dict = json.loads(raw_meas_str)
            packet = UM7AllRawPacket(**raw_meas_dict)
            logging.warning(f"[MF][RAW]: {packet}")
            sleep(3.0)  # move motors, do some hard work
        else:
            # here I need ot handle proc data
            proc_lock.acquire()
            proc_meas_bytes: bytes = proc_data_shm.buf[:]
            proc_lock.release()
            proc_meas_str = str(proc_meas_bytes, encoding='utf-8')
            proc_meas_dict = json.loads(proc_meas_str)
            packet = UM7AllProcPacket(**proc_meas_dict)
            logging.warning(f"[MF][PROC]: {packet}")
            sleep(2.0)  # move motors, do some hard work


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d]: %(message)s',
        datefmt='%H:%M:%S',
        handlers=[
            logging.FileHandler(f'{Path(__file__).stem}.log', mode='w'),
            logging.StreamHandler(sys.stdout),
        ])
    sensor_read_proc = Process(target=sensor_read_process, args=(raw_data_shm, proc_data_shm, raw_lock, proc_lock,))
    sensor_read_proc.start()

    sleep(1)
    main_function()

    proc_data_shm.close()
    proc_data_shm.unlink()

    raw_data_shm.close()
    raw_data_shm.unlink()



#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 3 May 2020

import logging
import os.path
import sys

from um7py.shearwater_serial import ShearWaterSerial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNHD.json")
    shearwater = ShearWaterSerial(device=device_file)

    print(f"\n========== COMMAND REGISTERS ===================================")
    print(f"get_fw_build_id               : {shearwater.get_fw_build_id}")
    print(f"get_fw_build_version          : {shearwater.get_fw_build_version}")
    print(f"board_unique_id_1             : {shearwater.board_unique_id_1}")
    print(f"board_unique_id_2             : {shearwater.board_unique_id_2}")
    print(f"protocol_version              : {shearwater.protocol_version}")




#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 27 November 2020


import logging
import os.path
import sys

from um7py.shearwater_serial import ShearWaterSerial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d]: %(message)s',
        datefmt='%H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log', mode='w'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNHD.json")
    shearwater = ShearWaterSerial(device=device_file)
    creg_com_rates5, *_ = shearwater.creg_com_rates5
    print("creg_com_rates5: {}".format(creg_com_rates5))
    creg_com_settings, *_ = shearwater.creg_com_settings
    print("um7 creg_com_settings: {}".format(creg_com_settings))

    print(f"setting new quat rate: 20 Hz")
    # look at the register description -->
    # https://docs.redshiftlabs.com.au/register_map_current.html#creg-com-rates5
    # we are going to set the quaternion transmission rate to 20 Hz
    creg_com_rates5.set_field_value(QUAT_RATE=20)
    # we have now set a raw register value, let us write it in the sensor
    print(creg_com_rates5)
    shearwater.creg_com_rates5 = creg_com_rates5.raw_value




#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# Date: 12 June 2020
# Version: v0.1
# License: MIT

from rsl_comm_py.rsl_spi import RslSpiUsbIss, RslSpiLinuxPort
from rsl_comm_py.um7_registers import UM7Registers


class UM7SpiLinuxPort(RslSpiLinuxPort, UM7Registers):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class UM7SpiUsbIss(RslSpiUsbIss, UM7Registers):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)


if __name__ == '__main__':
    um7_spi_iss = UM7SpiUsbIss()
    print(f"creg_com_settings             : {um7_spi_iss.creg_com_settings}")
    print(f"creg_com_rates1               : {um7_spi_iss.creg_com_rates1}")
    print(f"creg_com_rates2               : {um7_spi_iss.creg_com_rates2}")
    print(f"creg_com_rates3               : {um7_spi_iss.creg_com_rates3}")
    print(f"creg_com_rates4               : {um7_spi_iss.creg_com_rates4}")
    print(f"creg_com_rates5               : {um7_spi_iss.creg_com_rates5}")
    print(f"creg_com_rates6               : {um7_spi_iss.creg_com_rates6}")
    print(f"creg_com_rates7               : {um7_spi_iss.creg_com_rates7}")



#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 3 May 2020

import logging
import os.path
import sys

from um7py.shearwater_serial import ShearWaterSerial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNHD.json")
    shearwater = ShearWaterSerial(device=device_file)
    print(f"\n========== CONFIG REGISTERS ===================================")
    print(f"creg_com_settings             : {shearwater.creg_com_settings}")
    print(f"creg_com_rates1               : {shearwater.creg_com_rates1}")
    print(f"creg_com_rates2               : {shearwater.creg_com_rates2}")
    print(f"creg_com_rates3               : {shearwater.creg_com_rates3}")
    print(f"creg_com_rates4               : {shearwater.creg_com_rates4}")
    print(f"creg_com_rates5               : {shearwater.creg_com_rates5}")
    print(f"creg_com_rates6               : {shearwater.creg_com_rates6}")
    print(f"creg_com_rates7               : {shearwater.creg_com_rates7}")
    print(f"creg_misc_settings            : {shearwater.creg_misc_settings}")
    print(f"creg_gyro_1_meas_range        : {shearwater.creg_gyro_1_meas_range}")
    print(f"creg_gyro_1_trim_x            : {shearwater.creg_gyro_1_trim_x}")
    print(f"creg_gyro_1_trim_y            : {shearwater.creg_gyro_1_trim_y}")
    print(f"creg_gyro_1_trim_z            : {shearwater.creg_gyro_1_trim_z}")
    print(f"creg_gyro_2_meas_range        : {shearwater.creg_gyro_2_meas_range}")
    print(f"creg_gyro_2_trim_x            : {shearwater.creg_gyro_2_trim_x}")
    print(f"creg_gyro_2_trim_y            : {shearwater.creg_gyro_2_trim_y}")
    print(f"creg_gyro_2_trim_z            : {shearwater.creg_gyro_2_trim_z}")
    print(f"creg_mag_1_cal1_1             : {shearwater.creg_mag_1_cal1_1}")
    print(f"creg_mag_1_cal1_2             : {shearwater.creg_mag_1_cal1_2}")
    print(f"creg_mag_1_cal1_3             : {shearwater.creg_mag_1_cal1_3}")
    print(f"creg_mag_1_cal2_1             : {shearwater.creg_mag_1_cal2_1}")
    print(f"creg_mag_1_cal2_2             : {shearwater.creg_mag_1_cal2_2}")
    print(f"creg_mag_1_cal2_3             : {shearwater.creg_mag_1_cal2_3}")
    print(f"creg_mag_1_cal3_1             : {shearwater.creg_mag_1_cal3_1}")
    print(f"creg_mag_1_cal3_2             : {shearwater.creg_mag_1_cal3_2}")
    print(f"creg_mag_1_cal3_3             : {shearwater.creg_mag_1_cal3_3}")
    print(f"creg_mag_1_bias_x             : {shearwater.creg_mag_1_bias_x}")
    print(f"creg_mag_1_bias_y             : {shearwater.creg_mag_1_bias_y}")
    print(f"creg_mag_1_bias_z             : {shearwater.creg_mag_1_bias_z}")
    print(f"creg_mag_2_cal1_1             : {shearwater.creg_mag_2_cal1_1}")
    print(f"creg_mag_2_cal1_2             : {shearwater.creg_mag_2_cal1_2}")
    print(f"creg_mag_2_cal1_3             : {shearwater.creg_mag_2_cal1_3}")
    print(f"creg_mag_2_cal2_1             : {shearwater.creg_mag_2_cal2_1}")
    print(f"creg_mag_2_cal2_2             : {shearwater.creg_mag_2_cal2_2}")
    print(f"creg_mag_2_cal2_3             : {shearwater.creg_mag_2_cal2_3}")
    print(f"creg_mag_2_cal3_1             : {shearwater.creg_mag_2_cal3_1}")
    print(f"creg_mag_2_cal3_2             : {shearwater.creg_mag_2_cal3_2}")
    print(f"creg_mag_2_cal3_3             : {shearwater.creg_mag_2_cal3_3}")
    print(f"creg_mag_2_bias_x             : {shearwater.creg_mag_2_bias_x}")
    print(f"creg_mag_2_bias_y             : {shearwater.creg_mag_2_bias_y}")
    print(f"creg_mag_2_bias_z             : {shearwater.creg_mag_2_bias_z}")
    print(f"creg_accel_1_meas_range       : {shearwater.creg_accel_1_meas_range}")
    print(f"creg_accel_1_cal1_1           : {shearwater.creg_accel_1_cal1_1}")
    print(f"creg_accel_1_cal1_2           : {shearwater.creg_accel_1_cal1_2}")
    print(f"creg_accel_1_cal1_3           : {shearwater.creg_accel_1_cal1_3}")
    print(f"creg_accel_1_cal2_1           : {shearwater.creg_accel_1_cal2_1}")
    print(f"creg_accel_1_cal2_2           : {shearwater.creg_accel_1_cal2_2}")
    print(f"creg_accel_1_cal2_3           : {shearwater.creg_accel_1_cal2_3}")
    print(f"creg_accel_1_cal3_1           : {shearwater.creg_accel_1_cal3_1}")
    print(f"creg_accel_1_cal3_2           : {shearwater.creg_accel_1_cal3_2}")
    print(f"creg_accel_1_cal3_3           : {shearwater.creg_accel_1_cal3_3}")
    print(f"creg_accel_1_bias_x           : {shearwater.creg_accel_1_bias_x}")
    print(f"creg_accel_1_bias_y           : {shearwater.creg_accel_1_bias_y}")
    print(f"creg_accel_1_bias_z           : {shearwater.creg_accel_1_bias_z}")


#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 17 August 2020

import logging
import os.path
import sys
import json

from um7py.shearwater_serial import ShearWaterSerial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNHD.json")
    shearwater = ShearWaterSerial(device=device_file)
    creg_register_values = []
    _, uid_1 = shearwater.board_unique_id_1
    for reg in shearwater.svd_parser.cregs:
        name = reg.name.lower()
        reg, *_ = getattr(shearwater, name)
        creg_register_values.append(reg.as_dict())

    creg_register_str = json.dumps(creg_register_values, indent=2)

    with open(f'shearwater_creg_{uid_1:0X}.json', 'w') as fd:
        fd.write(creg_register_str)




#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 29 May 2020

import logging
import os.path
import sys

from um7py.um7_serial import UM7Serial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log', mode='w'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNP8.json")
    um7 = UM7Serial(device=device_file)
    print(f"\\n========== CONFIG REGISTERS ===================================")
    print(f"creg_com_settings             : {um7.creg_com_settings}")
    print(f"creg_com_rates1               : {um7.creg_com_rates1}")
    print(f"creg_com_rates2               : {um7.creg_com_rates2}")
    print(f"creg_com_rates3               : {um7.creg_com_rates3}")
    print(f"creg_com_rates4               : {um7.creg_com_rates4}")
    print(f"creg_com_rates5               : {um7.creg_com_rates5}")
    print(f"creg_com_rates6               : {um7.creg_com_rates6}")
    print(f"creg_com_rates7               : {um7.creg_com_rates7}")
    print(f"creg_misc_settings            : {um7.creg_misc_settings}")
    print(f"creg_home_north               : {um7.creg_home_north}")
    print(f"creg_home_east                : {um7.creg_home_east}")
    print(f"creg_home_up                  : {um7.creg_home_up}")
    print(f"creg_gyro_trim_x              : {um7.creg_gyro_trim_x}")
    print(f"creg_gyro_trim_y              : {um7.creg_gyro_trim_y}")
    print(f"creg_gyro_trim_z              : {um7.creg_gyro_trim_z}")
    print(f"creg_mag_cal1_1               : {um7.creg_mag_cal1_1}")
    print(f"creg_mag_cal1_2               : {um7.creg_mag_cal1_2}")
    print(f"creg_mag_cal1_3               : {um7.creg_mag_cal1_3}")
    print(f"creg_mag_cal2_1               : {um7.creg_mag_cal2_1}")
    print(f"creg_mag_cal2_2               : {um7.creg_mag_cal2_2}")
    print(f"creg_mag_cal2_3               : {um7.creg_mag_cal2_3}")
    print(f"creg_mag_cal3_1               : {um7.creg_mag_cal3_1}")
    print(f"creg_mag_cal3_2               : {um7.creg_mag_cal3_2}")
    print(f"creg_mag_cal3_3               : {um7.creg_mag_cal3_3}")
    print(f"creg_mag_bias_x               : {um7.creg_mag_bias_x}")
    print(f"creg_mag_bias_y               : {um7.creg_mag_bias_y}")
    print(f"creg_mag_bias_z               : {um7.creg_mag_bias_z}")
    print(f"creg_accel_cal1_1             : {um7.creg_accel_cal1_1}")
    print(f"creg_accel_cal1_2             : {um7.creg_accel_cal1_2}")
    print(f"creg_accel_cal1_3             : {um7.creg_accel_cal1_3}")
    print(f"creg_accel_cal2_1             : {um7.creg_accel_cal2_1}")
    print(f"creg_accel_cal2_2             : {um7.creg_accel_cal2_2}")
    print(f"creg_accel_cal2_3             : {um7.creg_accel_cal2_3}")
    print(f"creg_accel_cal3_1             : {um7.creg_accel_cal3_1}")
    print(f"creg_accel_cal3_2             : {um7.creg_accel_cal3_2}")
    print(f"creg_accel_cal3_3             : {um7.creg_accel_cal3_3}")
    print(f"creg_accel_bias_x             : {um7.creg_accel_bias_x}")
    print(f"creg_accel_bias_y             : {um7.creg_accel_bias_y}")
    print(f"creg_accel_bias_z             : {um7.creg_accel_bias_z}")



#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# Date: 2 March 2020
# Version: v0.1
# License: MIT

import json
import logging
import os
import os.path
import serial
import struct
import sys

from time import monotonic
from typing import Tuple, List, Dict, Any, Union, Callable

from rsl_comm_py.um7_broadcast_packets import UM7AllRawPacket, UM7HealthPacket, UM7GyroBiasPacket, UM7ProcMagPacket, \
    UM7ProcGyroPacket, UM7ProcAccelPacket, UM7RawMagPacket, UM7RawGyroPacket, UM7RawAccelPacket, UM7QuaternionPacket, \
    UM7EulerPacket, UM7AllProcPacket
from rsl_comm_py.um7_registers import UM7Registers


class RslException(Exception):
    """
    RSL Exception class for recording RedShiftLabs and/or UM7 specific errors
    """
    pass


class UM7Serial(UM7Registers):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.device_file = kwargs.get('device')
        self.device_dict = None
        self.port = serial.Serial()
        self.port_name = None
        self.port_config = None
        self.buffer = bytes()
        self.buffer_size = 125
        self.firmware_version = None
        self.uid_32_bit = None
        if kwargs.get('port_name') is not None:
            self.port_name = kwargs.get('port_name')
        else:
            port_found = self.find_port()
            if not port_found:
                raise RslException("Device specified in the config is not connected!")
        self.init_connection()

    def init_connection(self):
        self.port = serial.Serial(port=self.port_name)
        self.port.port = self.port_name
        self.port.baudrate = 115200
        if not self.port.is_open:
            self.port.open()

    def connect(self, *args, **kwargs):
        self.init_connection()

    def find_port(self):
        if not self.device_file:
            raise RslException("No configuration file specified!")
        if not os.path.isfile(self.device_file):
            src_path = os.path.dirname(__file__)
            guessing_file_path = os.path.join(src_path, self.device_file)
            if not os.path.isfile(guessing_file_path):
                raise RslException(f"Specify absolute path to the `device` json file, cannot find {self.device_file}")
            else:
                self.device_file = guessing_file_path
        with open(self.device_file) as fp:
            self.device_dict = json.load(fp)
        # go through each device and match vendor, then key
        if sys.platform.startswith('win32'):
            from serial.tools import list_ports
            device_list = list_ports.comports()
            for device in device_list:
                json_config = {}
                # go through device list and check each FTDI device for a match
                if device.manufacturer == "FTDI":
                    json_config['ID_VENDOR'] = device.manufacturer
                    json_config['ID_SERIAL_SHORT'] = device.serial_number
                    # don't know what the model should look like so were making it "pid:vid". 
                    # This matches the windows section of the autodetect script
                    json_config['ID_MODEL'] = '{0:04x}:{1:04x}'.format(device.vid, device.pid)
                    if self.device_dict == json_config:
                        self.port_name = device.device   # set serial port ("COM4", for example)
                        return True
            else:
                return False
        else:
            import pyudev
            context = pyudev.Context()
            for device in context.list_devices(subsystem='tty'):
                json_config = {}
                if device.get('ID_VENDOR') == 'FTDI':
                    for key in self.device_dict:
                        json_config[key] = device.get(key)
                    if self.device_dict == json_config:
                        self.port_name = device.device_node
                        return True
            else:
                return False

    def get_preamble(self):
        preamble = bytes('snp', encoding='ascii')
        return preamble

    def compute_checksum(self, partial_packet: bytes) -> bytes:
        checksum = 0
        for byte in partial_packet:
            checksum += byte
        checksum_bytes = int.to_bytes(checksum, length=2, byteorder='big', signed=False)
        return checksum_bytes

    def construct_packet_type(self, has_data: bool = False, is_batch: bool = False, data_length: int = 0,
                              hidden: bool = False, command_failed: bool = False) -> int:
        if data_length > 15:
            raise RslException(f"Batch size for command should not exceed 15, but got {data_length}")
        packet_type = has_data << 7 | is_batch << 6 | data_length << 2 | hidden << 1 | command_failed
        return packet_type

    def get_packet_type(self, extracted_packet_type: int) -> Tuple[bool, bool, int, bool, bool]:
        has_data = bool(extracted_packet_type >> 7 & 0x01)
        is_batch = bool(extracted_packet_type >> 6 & 0x01)
        batch_length = extracted_packet_type >> 2 & 0x0F
        hidden = bool(extracted_packet_type >> 1 & 0x01)
        command_failed = bool(extracted_packet_type & 0x01)
        return has_data, is_batch, batch_length, hidden, command_failed

    def construct_packet(self, packet_type: int, address: int, payload: bytes = bytes()) -> bytes:
        preamble = self.get_preamble()
        packet_type_byte = int.to_bytes(packet_type, length=1, byteorder='big')
        address_byte = int.to_bytes(address, length=1, byteorder='big')
        partial_packet = preamble + packet_type_byte + address_byte + payload
        checksum = self.compute_checksum(partial_packet)
        packet = partial_packet + checksum
        return packet

    def send(self, packet: bytes) -> bool:
        bytes_written = self.port.write(packet)
        self.port.flush()
        return bytes_written == len(packet)

    def recv(self) -> Tuple[bool, bytes]:
        ok = False
        while not ok:
            # read until we get something in the buffer
            in_waiting = self.port.in_waiting
            logging.info(f"waiting buffer: {in_waiting}")
            self.buffer += self.port.read(self.buffer_size)
            # self.buffer += self.port.read(in_waiting)
            logging.info(f"buffer size: {len(self.buffer)}")
            # self.__buffer = self.__port.read(self.__port.inWaiting()) # causes too long of a delay
            ok = len(self.buffer) > 0
        return True, self.buffer

    def send_recv(self, packet: bytes) -> bytes:
        send_ok = self.send(packet)
        if not send_ok:
            raise RslException("Sending packet failed!")
        recv_ok, _ = self.recv()
        if not recv_ok:
            raise RslException("Receiving packet failed!")
        return self.buffer

    def find_packet(self, sensor_response: bytes) -> Tuple[bytes, bytes]:
        preamble = self.get_preamble()
        packet_start_idx = sensor_response.find(preamble)

        if packet_start_idx == -1:
            # preamble of packet not found, exit
            return bytes(), bytes()

        next_packet_rel_idx = sensor_response[packet_start_idx+3:].find(preamble)
        next_packet_start_idx = packet_start_idx + 3 + next_packet_rel_idx

        if next_packet_start_idx == -1:
            # preamble of next packet not found, exit
            return bytes(), bytes()

        # complete packet found in data
        packet = sensor_response[packet_start_idx:next_packet_start_idx]
        remainder = sensor_response[next_packet_start_idx:]
        return packet, remainder

    def find_response(self, reg_addr: int, hidden: bool = False, expected_length: int = 7) -> Tuple[bool, bytes]:
        while len(self.buffer) > 0:
            packet, self.buffer = self.find_packet(self.buffer)
            if len(packet) < 4:
                return False, bytes()
            logging.debug(f"{packet}")
            logging.debug(f"addr: {packet[4]}")
            packet_type = packet[3]
            is_packet_hidden = bool((packet_type >> 1) & 0x01)
            response_addr = packet[4]
            if response_addr == reg_addr and hidden == is_packet_hidden and len(packet) == expected_length:
                # required packet found
                return True, packet
        else:
            return False, bytes()

    def verify_checksum(self, packet: bytes) -> bool:
        computed_checksum = 0
        for byte in packet[:-2]:
            computed_checksum += byte
        received_checksum = int.from_bytes(packet[-2:], byteorder='big', signed=False)
        return computed_checksum == received_checksum

    def get_payload(self, packet: bytes) -> Tuple[bool, bytes]:
        ok = self.verify_checksum(packet)
        if not ok:
            raise RslException("Packet checksum INVALID!")
        return True, packet[5:-2]

    def check_packet(self, packet: bytes) -> bool:
        packet_type = packet[3]
        has_data = bool((packet_type >> 7) & 0x01)
        is_batch = bool((packet_type >> 6) & 0x01)
        data_len = (packet_type >> 2) & 0x0F
        hidden = bool((packet_type >> 1) & 0x01)
        error = packet_type & 0x01
        if error:
            logging.error(f"Error bit set for packet: {packet}!")
            return False
        elif not has_data and len(packet) != 7:
            logging.error(f"Packet without data (has_data = 0) shall have 7 bytes, got {len(packet)}")
            return False
        elif has_data and data_len == 0 and len(packet) != 11:
            logging.error(f"Single packet has 4 bytes payload, in total 11 bytes, got {len(packet)}")
            return False
        elif has_data and data_len > 0 and len(packet) != 7 + 4 * data_len:
            logging.error(f"Batch packet with data_len {data_len} shall be {7 + 4 * data_len} bytes, got {len(packet)}")
            return False
        else:
            # all the checks pass then
            return True

    def read_register(self, reg_addr: int, hidden: bool = False) -> Tuple[bool, bytes]:
        packet_type = self.construct_packet_type(hidden=hidden)
        packet_to_send = self.construct_packet(packet_type, reg_addr)
        logging.debug(f"packet sent: {packet_to_send}")
        self.send_recv(packet_to_send)
        t = monotonic()
        ok, sensor_reply = self.find_response(reg_addr, hidden, 11)
        if ok:
            logging.debug(f"packet: {sensor_reply}")
            self.check_packet(sensor_reply)
            ok, payload = self.get_payload(sensor_reply)
            return ok, payload
        else:
            while monotonic() - t < 0.2:
                # try to send <-> receive packets for a pre-defined time out time
                self.send_recv(packet_to_send)
                ok, sensor_reply = self.find_response(reg_addr, hidden, 11)
                if ok and len(sensor_reply) > 7:
                    logging.debug(f"packet: {sensor_reply}")
                    self.check_packet(sensor_reply)
                    ok, payload = self.get_payload(sensor_reply)
                    return ok, payload
            return False, bytes()

    def write_register(self, reg_addr: int, reg_value: Union[int, bytes, float], hidden: bool = False) -> bool:
        packet_type = self.construct_packet_type(has_data=True, hidden=hidden)
        if type(reg_value) == int:
            payload = int.to_bytes(reg_value, byteorder='big', length=4)
        elif type(reg_value) == bytes:
            payload = reg_value
        elif type(reg_value) == float:
            payload = struct.pack('>f', reg_value)
        packet_to_send = self.construct_packet(packet_type, reg_addr, payload)
        logging.debug(f"packet sent: {packet_to_send}")
        self.send_recv(packet_to_send)
        ok, sensor_reply = self.find_response(reg_addr)
        if ok:
            logging.debug(f"packet: {sensor_reply}")
            self.check_packet(sensor_reply)
            ok, payload = self.get_payload(sensor_reply)
            return ok

    def recv_broadcast_packet(self, packet_target_addr: int, expected_packet_length: int,
                              decode_callback: Callable, num_packets: int = -1):
        received_packets = 0
        while num_packets == -1 or received_packets < num_packets:
            ok, _ = self.recv()
            while len(self.buffer) > 0:
                packet, self.buffer = self.find_packet(self.buffer)
                # logging.info(f"buffer length: {len(self.buffer)}")
                if len(packet) > 7:
                    recv_packet_addr = packet[4]
                    if recv_packet_addr == packet_target_addr:
                        packet_correct_length = len(packet) == expected_packet_length
                        if not packet_correct_length:
                            logging.error(f"Invalid packet length for addr: {packet_target_addr}, "
                                          f"expected: {expected_packet_length}, got: {len(packet)}, packet: {packet}")
                        checksum_ok = self.verify_checksum(packet)
                        if not checksum_ok:
                            logging.error(f"Checksum failed for broadcast packet with addr: {packet_target_addr}")
                        packet_type_check_ok = self.check_packet(packet)
                        if not packet_type_check_ok:
                            logging.error(f"Checking packet type failed for broadcast with addr: {packet_target_addr}!")
                        if packet_correct_length and checksum_ok and packet_type_check_ok:
                            yield decode_callback(packet)
                            received_packets += 1

    def recv_all_raw_broadcast(self, num_packets: int = -1):
        all_raw_start_addr = self.svd_parser.find_register_by(name='DREG_GYRO_RAW_XY').address
        broadcast_packet_length = 51
        return self.recv_broadcast_packet(all_raw_start_addr, broadcast_packet_length,
                                          self.decode_all_raw_broadcast, num_packets)

    def recv_all_proc_broadcast(self, num_packets: int = -1):
        all_proc_start_addr = self.svd_parser.find_register_by(name='DREG_GYRO_PROC_X').address
        broadcast_packet_length = 55
        return self.recv_broadcast_packet(all_proc_start_addr, broadcast_packet_length,
                                          self.decode_all_proc_broadcast, num_packets)

    def recv_euler_broadcast(self, num_packets: int = -1):
        euler_start_addr = self.svd_parser.find_register_by(name='DREG_EULER_PHI_THETA').address
        broadcast_packet_length = 27
        return self.recv_broadcast_packet(euler_start_addr, broadcast_packet_length,
                                          self.decode_euler_broadcast, num_packets)

    def recv_quaternion_broadcast(self, num_packets: int = -1):
        quat_start_addr = self.svd_parser.find_register_by(name='DREG_QUAT_AB').address
        broadcast_packet_length = 19
        return self.recv_broadcast_packet(quat_start_addr, broadcast_packet_length,
                                          self.decode_quaternion_broadcast, num_packets)

    def recv_health_broadcast(self, num_packets: int = -1):
        health_start_addr = self.svd_parser.find_register_by(name='DREG_HEALTH').address
        broadcast_packet_length = 11
        return self.recv_broadcast_packet(health_start_addr, broadcast_packet_length,
                                          self.decode_health_broadcast, num_packets)

    def recv_raw_accel_broadcast(self, num_packets: int = -1):
        raw_accel_1_addr = self.svd_parser.find_register_by(name='DREG_ACCEL_RAW_XY').address
        broadcast_packet_length = 19
        return self.recv_broadcast_packet(raw_accel_1_addr, broadcast_packet_length,
                                          self.decode_raw_accel_broadcast, num_packets)

    def recv_raw_gyro_broadcast(self, num_packets: int = -1):
        raw_gyro_1_addr = self.svd_parser.find_register_by(name='DREG_GYRO_RAW_XY').address
        broadcast_packet_length = 19
        return self.recv_broadcast_packet(raw_gyro_1_addr, broadcast_packet_length,
                                          self.decode_raw_gyro_broadcast, num_packets)

    def recv_raw_mag_broadcast(self, num_packets: int = -1):
        raw_mag_1_addr = self.svd_parser.find_register_by(name='DREG_MAG_RAW_XY').address
        broadcast_packet_length = 19
        return self.recv_broadcast_packet(raw_mag_1_addr, broadcast_packet_length,
                                          self.decode_raw_mag_broadcast, num_packets)

    def recv_proc_accel_broadcast(self, num_packets: int = -1):
        proc_accel_1_addr = self.svd_parser.find_register_by(name='DREG_ACCEL_PROC_X').address
        broadcast_packet_length = 23
        return self.recv_broadcast_packet(proc_accel_1_addr, broadcast_packet_length,
                                          self.decode_proc_accel_broadcast, num_packets)

    def recv_proc_gyro_broadcast(self, num_packets: int = -1):
        proc_gyro_1_addr = self.svd_parser.find_register_by(name='DREG_GYRO_PROC_X').address
        broadcast_packet_length = 23
        return self.recv_broadcast_packet(proc_gyro_1_addr, broadcast_packet_length,
                                          self.decode_proc_gyro_broadcast, num_packets)

    def recv_proc_mag_broadcast(self, num_packets: int = -1):
        proc_mag_1_addr = self.svd_parser.find_register_by(name='DREG_MAG_PROC_X').address
        broadcast_packet_length = 23
        return self.recv_broadcast_packet(proc_mag_1_addr, broadcast_packet_length,
                                          self.decode_proc_mag_broadcast, num_packets)

    def recv_broadcast(self,  num_packets: int = -1):
        received_packets = 0
        while num_packets == -1 or received_packets < num_packets:
            ok, _ = self.recv()
            while len(self.buffer) > 0:
                packet, self.buffer = self.find_packet(self.buffer)
                if len(packet) > 7:
                    packet_addr = packet[4]
                    start_reg = self.svd_parser.find_register_by(address=packet_addr)
                    checksum_ok = self.verify_checksum(packet)
                    if not checksum_ok:
                        logging.error(f"Checksum failed for broadcast packet with start_reg: {start_reg}")
                    packet_type_check_ok = self.check_packet(packet)
                    if not packet_type_check_ok:
                        logging.error(f"Checking packet type failed for broadcast with start_reg: {start_reg}!")
                    health_start_addr = self.svd_parser.find_register_by(name='DREG_HEALTH').address
                    euler_start_addr = self.svd_parser.find_register_by(name='DREG_EULER_PHI_THETA').address
                    all_proc_start_addr = self.svd_parser.find_register_by(name='DREG_GYRO_PROC_X').address
                    accel_proc_start_addr = self.svd_parser.find_register_by(name='DREG_ACCEL_PROC_X').address
                    mag_proc_start_addr = self.svd_parser.find_register_by(name='DREG_MAG_PROC_X').address
                    all_raw_start_addr = self.svd_parser.find_register_by(name='DREG_GYRO_RAW_XY').address
                    accel_raw_start_addr = self.svd_parser.find_register_by(name='DREG_ACCEL_RAW_XY').address
                    mag_raw_start_addr = self.svd_parser.find_register_by(name='DREG_MAG_RAW_XY').address
                    gyro_bias_start_addr = self.svd_parser.find_register_by(name='DREG_GYRO_BIAS_X').address
                    quat_addr = self.svd_parser.find_register_by(name='DREG_QUAT_AB').address

                    if packet_addr == health_start_addr:
                        if len(packet) == 11:
                            logging.info("[HEALTH]: broadcast packet found!")
                            yield self.decode_health_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[HEALTH]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    elif packet_addr == euler_start_addr:
                        if len(packet) == 27:
                            logging.info("[EULER]: broadcast packet found!")
                            yield self.decode_euler_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[EULER]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    elif packet_addr == all_proc_start_addr:
                        if len(packet) == 55:
                            logging.info("[ALL_PROC]: broadcast packet found!")
                            yield self.decode_all_proc_broadcast(packet)
                            received_packets += 1
                        elif len(packet) == 23:
                            logging.info("[GYRO_PROC]: broadcast packet found!")
                            yield self.decode_proc_gyro_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[GYRO_PROC]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    elif packet_addr == accel_proc_start_addr:
                        if len(packet) == 23:
                            logging.info("[ACCEL_PROC]: broadcast packet found!")
                            yield self.decode_proc_accel_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[ACCEL_PROC]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    elif packet_addr == mag_proc_start_addr:
                        if len(packet) == 23:
                            logging.info("[MAG_PROC]: broadcast packet found!")
                            yield self.decode_proc_mag_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[MAG_PROC]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    elif packet_addr == all_raw_start_addr:
                        if len(packet) == 51:
                            logging.info("[ALL_RAW]: broadcast packet found!")
                            yield self.decode_all_raw_broadcast(packet)
                            received_packets += 1
                        elif len(packet) == 19:
                            logging.info("[GYRO_RAW]: broadcast packet found!")
                            yield self.decode_raw_gyro_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[GYRO_RAW]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    elif packet_addr == accel_raw_start_addr:
                        if len(packet) == 19:
                            logging.info("[ACCEL_RAW]: broadcast packet found!")
                            yield self.decode_raw_accel_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[ACCEL_RAW]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    elif packet_addr == mag_raw_start_addr:
                        if len(packet) == 23:
                            logging.info("[MAG_RAW]: broadcast packet found!")
                            yield self.decode_raw_mag_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[MAG_RAW]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    elif packet_addr == quat_addr:
                        if len(packet) == 19:
                            logging.info("[QUAT]: quaternion broadcast packet found!")
                            yield self.decode_quaternion_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[QUAT]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    elif packet_addr == gyro_bias_start_addr:
                        if len(packet) == 19:
                            logging.info("[GYRO_1_BIAS]: broadcast packet found!")
                            yield self.decode_gyro_bias_broadcast(packet)
                            received_packets += 1
                        else:
                            logging.error(f"[GYRO_1_BIAS]: invalid packet length, got {len(packet)}, packet: {packet}!")
                    else:
                        logging.error(f"[BROADCAST ERROR]: packet with addr {packet_addr}, reg: {start_reg.name} found "
                                      f"of length: {len(packet)} bytes, "
                                      f"no decoding is implemented for this!! Packet: {packet}")

    def decode_all_raw_broadcast(self, packet) -> UM7AllRawPacket:
        payload = packet[5:-2]
        g_x, g_y, g_z, g_time = struct.unpack('>hhh2xf', payload[0:12])
        a_x, a_y, a_z, a_time = struct.unpack('>hhh2xf', payload[12:24])
        m_x, m_y, m_z, m_time = struct.unpack('>hhh2xf',   payload[24:36])
        T, T_t = struct.unpack('>ff', payload[36:44])
        return UM7AllRawPacket(gyro_raw_x=g_x, gyro_raw_y=g_y, gyro_raw_z=g_z, gyro_raw_time=g_time,
                               accel_raw_x=a_x, accel_raw_y=a_y, accel_raw_z=a_z, accel_raw_time=a_time,
                               mag_raw_x=m_x, mag_raw_y=m_y, mag_raw_z=m_z, mag_raw_time=m_time,
                               temperature=T, temperature_time=T_t)

    def decode_all_proc_broadcast(self, packet) -> UM7AllProcPacket:
        payload = packet[5:-2]
        g_x, g_y, g_z, g_time = struct.unpack('>ffff', payload[0:16])
        a_x, a_y, a_z, a_time = struct.unpack('>ffff', payload[16:32])
        m_x, m_y, m_z, m_time = struct.unpack('>ffff', payload[32:48])
        return UM7AllProcPacket(gyro_proc_x=g_x, gyro_proc_y=g_y, gyro_proc_z=g_z, gyro_proc_time=g_time,
                                accel_proc_x=a_x, accel_proc_y=a_y, accel_proc_z=a_z, accel_proc_time=a_time,
                                mag_proc_x=m_x, mag_proc_y=m_y, mag_proc_z=m_z, mag_proc_time=m_time)

    def decode_euler_broadcast(self, packet) -> UM7EulerPacket:
        payload = packet[5:-2]
        roll, pitch, yaw = struct.unpack('>hhh2x', payload[0:8])
        roll_rate, pitch_rate, yaw_rate, time_stamp = struct.unpack('>hhh2xf', payload[8:20])
        return UM7EulerPacket(
            roll=roll/91.02222, pitch=pitch/91.02222, yaw=yaw/91.02222,
            roll_rate=roll_rate/91.02222, pitch_rate=pitch_rate/91.02222, yaw_rate=yaw_rate/91.02222,
            time_stamp=time_stamp
        )

    def decode_quaternion_broadcast(self, packet) -> UM7QuaternionPacket:
        payload = packet[5:-2]
        q_w, q_x, q_y, q_z, q_time = struct.unpack('>hhhhf', payload[0:12])
        return UM7QuaternionPacket(
            q_w=q_w/29789.09091, q_x=q_x/29789.09091, q_y=q_y/29789.09091, q_z=q_z/29789.09091, q_time=q_time
        )

    def decode_raw_accel_broadcast(self, packet) -> UM7RawAccelPacket:
        payload = packet[5:-2]
        a_x, a_y, a_z, a_time = struct.unpack('>hhh2xf', payload[0:12])
        return UM7RawAccelPacket(accel_raw_x=a_x, accel_raw_y=a_y, accel_raw_z=a_z, accel_raw_time=a_time)

    def decode_raw_gyro_broadcast(self, packet) -> UM7RawGyroPacket:
        payload = packet[5:-2]
        g_x, g_y, g_z, g_time = struct.unpack('>hhh2xf', payload[0:12])
        return UM7RawGyroPacket(gyro_raw_x=g_x, gyro_raw_y=g_y, gyro_raw_z=g_z, gyro_raw_time=g_time)

    def decode_raw_mag_broadcast(self, packet) -> UM7RawMagPacket:
        payload = packet[5:-2]
        m_x, m_y, m_z, m_time = struct.unpack('>hhh2xf', payload[0:12])
        return UM7RawMagPacket(mag_raw_x=m_x, mag_raw_y=m_y, mag_raw_z=m_z, mag_raw_time=m_time)

    def decode_proc_accel_broadcast(self, packet) -> UM7ProcAccelPacket:
        payload = packet[5:-2]
        a_x, a_y, a_z, a_time = struct.unpack('>ffff', payload[0:16])
        return UM7ProcAccelPacket(accel_proc_x=a_x, accel_proc_y=a_y, accel_proc_z=a_z, accel_proc_time=a_time)

    def decode_proc_gyro_broadcast(self, packet) -> UM7ProcGyroPacket:
        payload = packet[5:-2]
        g_x, g_y, g_z, g_time = struct.unpack('>ffff', payload[0:16])
        return UM7ProcGyroPacket(gyro_proc_x=g_x, gyro_proc_y=g_y, gyro_proc_z=g_z, gyro_proc_time=g_time)

    def decode_proc_mag_broadcast(self, packet) -> UM7ProcMagPacket:
        payload = packet[5:-2]
        m_x, m_y, m_z, m_time = struct.unpack('>ffff', payload[0:16])
        return UM7ProcMagPacket(mag_proc_x=m_x, mag_proc_y=m_y, mag_proc_z=m_z, mag_proc_time=m_time)

    def decode_gyro_bias_broadcast(self, packet) -> UM7GyroBiasPacket:
        payload = packet[5:-2]
        gyro_bias_x, gyro_bias_y, gyro_bias_z, = struct.unpack('>fff', payload[0:12])
        return UM7GyroBiasPacket(gyro_bias_x=gyro_bias_x, gyro_bias_y=gyro_bias_y, gyro_bias_z=gyro_bias_z)

    def decode_health_broadcast(self, packet) -> UM7HealthPacket:
        payload = packet[5:-2]
        health, = struct.unpack('>I', payload[0:4])
        return UM7HealthPacket(health=health)

    def hidden_regs_values(self) -> List[Dict]:
        hidden_regs_as_json = []
        for reg in self.svd_parser.hidden_regs:
            name = reg.name.lower()
            reg, *_ = getattr(self, name)
            hidden_regs_as_json.append(reg.as_dict())
        return hidden_regs_as_json

    def creg_regs_values(self) -> List[Dict]:
        config_regs_as_json = []
        for reg in self.svd_parser.cregs:
            name = reg.name.lower()
            reg, *_ = getattr(self, name)
            config_regs_as_json.append(reg.as_dict())
        return config_regs_as_json

if __name__ == '__main__':
    pass
    um7 = UM7Serial()



from rsl_comm_py.um7_autodetect import um7_autodetect
from rsl_comm_py.serve_um7_autodetect import serve_autodetect_script
from rsl_comm_py.shearwater_serial import ShearWaterSerial
from rsl_comm_py.shearwater_spi import ShearWaterSpiUsbIss, ShearWaterSpiLinuxPort
from rsl_comm_py.um7_serial import UM7Serial
from rsl_comm_py.um7_spi import UM7SpiUsbIss, UM7SpiLinuxPort



#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 29 May 2020

import logging
import os.path
import sys

from um7py.um7_serial import UM7Serial


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.WARNING,
        format='[%(asctime)s.%(msecs)03d] [%(levelname)-8s]:  %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
        handlers=[
            logging.FileHandler(f'{os.path.basename(__file__)}.log', mode='w'),
            logging.StreamHandler(sys.stdout),
        ])
    script_dir = os.path.dirname(__file__)
    device_file = os.path.join(script_dir, os.pardir, "um7py", "um7_A500CNP8.json")
    um7 = UM7Serial(device=device_file)

    print(f"\\n========== DATA REGISTERS ===================================")
    print(f"dreg_health                   : {um7.dreg_health}")
    print(f"dreg_gyro_raw_xy              : {um7.dreg_gyro_raw_xy}")
    print(f"dreg_gyro_raw_z               : {um7.dreg_gyro_raw_z}")
    print(f"dreg_gyro_raw_time            : {um7.dreg_gyro_raw_time}")
    print(f"dreg_accel_raw_xy             : {um7.dreg_accel_raw_xy}")
    print(f"dreg_accel_raw_z              : {um7.dreg_accel_raw_z}")
    print(f"dreg_accel_raw_time           : {um7.dreg_accel_raw_time}")
    print(f"dreg_mag_raw_xy               : {um7.dreg_mag_raw_xy}")
    print(f"dreg_mag_raw_z                : {um7.dreg_mag_raw_z}")
    print(f"dreg_mag_raw_time             : {um7.dreg_mag_raw_time}")
    print(f"dreg_temperature              : {um7.dreg_temperature}")
    print(f"dreg_temperature_time         : {um7.dreg_temperature_time}")
    print(f"dreg_gyro_proc_x              : {um7.dreg_gyro_proc_x}")
    print(f"dreg_gyro_proc_y              : {um7.dreg_gyro_proc_y}")
    print(f"dreg_gyro_proc_z              : {um7.dreg_gyro_proc_z}")
    print(f"dreg_gyro_proc_time           : {um7.dreg_gyro_proc_time}")
    print(f"dreg_accel_proc_x             : {um7.dreg_accel_proc_x}")
    print(f"dreg_accel_proc_y             : {um7.dreg_accel_proc_y}")
    print(f"dreg_accel_proc_z             : {um7.dreg_accel_proc_z}")
    print(f"dreg_accel_proc_time          : {um7.dreg_accel_proc_time}")
    print(f"dreg_mag_proc_x               : {um7.dreg_mag_proc_x}")
    print(f"dreg_mag_proc_y               : {um7.dreg_mag_proc_y}")
    print(f"dreg_mag_proc_z               : {um7.dreg_mag_proc_z}")
    print(f"dreg_mag_proc_time            : {um7.dreg_mag_proc_time}")
    print(f"dreg_quat_ab                  : {um7.dreg_quat_ab}")
    print(f"dreg_quat_cd                  : {um7.dreg_quat_cd}")
    print(f"dreg_quat_time                : {um7.dreg_quat_time}")
    print(f"dreg_euler_phi_theta          : {um7.dreg_euler_phi_theta}")
    print(f"dreg_euler_psi                : {um7.dreg_euler_psi}")
    print(f"dreg_euler_phi_theta_dot      : {um7.dreg_euler_phi_theta_dot}")
    print(f"dreg_euler_psi_dot            : {um7.dreg_euler_psi_dot}")
    print(f"dreg_euler_time               : {um7.dreg_euler_time}")
    print(f"dreg_position_north           : {um7.dreg_position_north}")
    print(f"dreg_position_east            : {um7.dreg_position_east}")
    print(f"dreg_position_up              : {um7.dreg_position_up}")
    print(f"dreg_position_time            : {um7.dreg_position_time}")
    print(f"dreg_velocity_north           : {um7.dreg_velocity_north}")
    print(f"dreg_velocity_east            : {um7.dreg_velocity_east}")
    print(f"dreg_velocity_up              : {um7.dreg_velocity_up}")
    print(f"dreg_velocity_time            : {um7.dreg_velocity_time}")
    print(f"dreg_gps_latitude             : {um7.dreg_gps_latitude}")
    print(f"dreg_gps_longitude            : {um7.dreg_gps_longitude}")
    print(f"dreg_gps_altitude             : {um7.dreg_gps_altitude}")
    print(f"dreg_gps_course               : {um7.dreg_gps_course}")
    print(f"dreg_gps_speed                : {um7.dreg_gps_speed}")
    print(f"dreg_gps_time                 : {um7.dreg_gps_time}")
    print(f"dreg_gps_sat_1_2              : {um7.dreg_gps_sat_1_2}")
    print(f"dreg_gps_sat_3_4              : {um7.dreg_gps_sat_3_4}")
    print(f"dreg_gps_sat_5_6              : {um7.dreg_gps_sat_5_6}")
    print(f"dreg_gps_sat_7_8              : {um7.dreg_gps_sat_7_8}")
    print(f"dreg_gps_sat_9_10             : {um7.dreg_gps_sat_9_10}")
    print(f"dreg_gps_sat_11_12            : {um7.dreg_gps_sat_11_12}")
    print(f"dreg_gyro_bias_x              : {um7.dreg_gyro_bias_x}")
    print(f"dreg_gyro_bias_y              : {um7.dreg_gyro_bias_y}")
    print(f"dreg_gyro_bias_z              : {um7.dreg_gyro_bias_z}")




#!/usr/bin/env python3
# Author: Dr. Konstantin Selyunin
# License: MIT
# Date: 3 May 2020
# Modified: 26 September 2020


from dataclasses import dataclass


@dataclass
class ShearWaterAllRawPacket:
    gyro_1_raw_x: int
    gyro_1_raw_y: int
    gyro_1_raw_z: int
    gyro_1_raw_time: float
    gyro_2_raw_x: int
    gyro_2_raw_y: int
    gyro_2_raw_z: int
    gyro_2_raw_time: float
    accel_1_raw_x: int
    accel_1_raw_y: int
    accel_1_raw_z: int
    accel_1_raw_time: float
    mag_1_raw_x: int
    mag_1_raw_y: int
    mag_1_raw_z: int
    mag_1_raw_time: float
    mag_2_raw_x: int
    mag_2_raw_y: int
    mag_2_raw_z: int
    mag_2_raw_time: float
    temperature: float
    temperature_time: float

    def __repr__(self):
        return f"RawPacket("\
               f"gyro_1=[{self.gyro_1_raw_x:>+5d}, {self.gyro_1_raw_y:>+5d}, {self.gyro_1_raw_z:>+5d}], "\
               f"gyro_1_t={self.gyro_1_raw_time:>6.3f}; " \
               f"gyro_2=[{self.gyro_2_raw_x:>+5d}, {self.gyro_2_raw_y:>+5d}, {self.gyro_2_raw_z:>+5d}], " \
               f"gyro_2_t={self.gyro_2_raw_time:>6.3f}; " \
               f"accel_1=[{self.accel_1_raw_x:>+5d}, {self.accel_1_raw_y:>+5d}, {self.accel_1_raw_z:>+5d}], " \
               f"accel_1_t={self.accel_1_raw_time:>6.3f}; " \
               f"mag_1=[{self.mag_1_raw_x:>+8d}, {self.mag_1_raw_y:>+8d}, {self.mag_1_raw_z:>+8d}], " \
               f"mag_1_t={self.mag_1_raw_time:>6.3f}; " \
               f"mag_2=[{self.mag_2_raw_x:>+5d}, {self.mag_2_raw_y:>+5d}, {self.mag_2_raw_z:>+5d}], " \
               f"mag_2_t={self.mag_2_raw_time:>6.3f}; " \
               f"T={self.temperature:>+3.2f}, " \
               f"T_t={self.temperature_time:>6.3f})"

    def to_csv(self):
        return f"{self.gyro_1_raw_x:>+5d};{self.gyro_1_raw_y:>+5d};{self.gyro_1_raw_z:>+5d};" \
               f"{self.gyro_1_raw_time:>6.5f};" \
               f"{self.gyro_2_raw_x:>+5d};{self.gyro_2_raw_y:>+5d};{self.gyro_2_raw_z:>+5d};" \
               f"{self.gyro_2_raw_time:>6.5f};" \
               f"{self.accel_1_raw_x:>+5d};{self.accel_1_raw_y:>+5d};{self.accel_1_raw_z:>+5d};" \
               f"{self.accel_1_raw_time:>6.5f};" \
               f"{self.mag_1_raw_x:>+8d};{self.mag_1_raw_y:>+8d};{self.mag_1_raw_z:>+8d};" \
               f"{self.mag_1_raw_time:>6.5f};" \
               f"{self.mag_2_raw_x:>+5d};{self.mag_2_raw_y:>+5d};{self.mag_2_raw_z:>+5d};" \
               f"{self.mag_2_raw_time:>6.5f};" \
               f"{self.temperature:>+3.2f};" \
               f"{self.temperature_time:>6.5f};\n"

    @staticmethod
    def csv_header():
        return "gyro_1_raw_x;gyro_1_raw_y;gyro_1_raw_z;gyro_1_raw_time;" \
               "gyro_2_raw_x;gyro_2_raw_y;gyro_2_raw_z;gyro_2_raw_time;" \
               "accel_1_raw_x;accel_1_raw_y;accel_1_raw_z;accel_1_raw_time;" \
               "mag_1_raw_x;mag_1_raw_y;mag_1_raw_z;mag_1_raw_time;" \
               "mag_2_raw_x;mag_2_raw_y;mag_2_raw_z;mag_2_raw_time;" \
               "temperature;temperature_time;\n"


@dataclass
class ShearWaterAllProcPacket:
    gyro_1_proc_x: float
    gyro_1_proc_y: float
    gyro_1_proc_z: float
    gyro_1_proc_time: float
    gyro_2_proc_x: float
    gyro_2_proc_y: float
    gyro_2_proc_z: float
    gyro_2_proc_time: float
    accel_1_proc_x: float
    accel_1_proc_y: float
    accel_1_proc_z: float
    accel_1_proc_time: float
    mag_1_proc_x: float
    mag_1_proc_y: float
    mag_1_proc_z: float
    mag_1_norm: float
    mag_1_proc_time: float
    mag_2_proc_x: float
    mag_2_proc_y: float
    mag_2_proc_z: float
    mag_2_norm: float
    mag_2_proc_time: float

    def __repr__(self):
        return f"ProcPacket("\
               f"gyro_1=[{self.gyro_1_proc_x:>+8.3f}, {self.gyro_1_proc_y:>+8.3f}, {self.gyro_1_proc_z:>+8.3f}], "\
               f"gyro_1_t={self.gyro_1_proc_time:>6.3f}; " \
               f"gyro_2=[{self.gyro_2_proc_x:>+8.3f}, {self.gyro_2_proc_y:>+8.3f}, {self.gyro_2_proc_z:>+8.3f}], " \
               f"gyro_2_t={self.gyro_2_proc_time:>6.3f}; " \
               f"accel_1=[{self.accel_1_proc_x:>+8.3f}, {self.accel_1_proc_y:>+8.3f}, {self.accel_1_proc_z:>+8.3f}], " \
               f"accel_1_t={self.accel_1_proc_time:>6.3f}; " \
               f"mag_1=[{self.mag_1_proc_x:>+8.6f}, {self.mag_1_proc_y:>+8.6f}, {self.mag_1_proc_z:>+8.6f}], " \
               f"mag_1_norm={self.mag_1_norm:>+8.6f}, " \
               f"mag_1_t={self.mag_1_proc_time:>6.3f}; " \
               f"mag_2=[{self.mag_2_proc_x:>+8.3f}, {self.mag_2_proc_y:>+8.3f}, {self.mag_2_proc_z:>+8.3f}], " \
               f"mag_2_norm={self.mag_2_norm:>+8.3f}, " \
               f"mag_2_t={self.mag_2_proc_time:>6.3f})"

    def to_csv(self):
        return f"{self.gyro_1_proc_x:>+8.3f};{self.gyro_1_proc_y:>+8.3f};{self.gyro_1_proc_z:>+8.3f};" \
               f"{self.gyro_1_proc_time:>6.5f};" \
               f"{self.gyro_2_proc_x:>+8.3f};{self.gyro_2_proc_y:>+8.3f};{self.gyro_2_proc_z:>+8.3f};" \
               f"{self.gyro_2_proc_time:>6.5f};" \
               f"{self.accel_1_proc_x:>+8.3f};{self.accel_1_proc_y:>+8.3f};{self.accel_1_proc_z:>+8.3f};" \
               f"{self.accel_1_proc_time:>6.5f};" \
               f"{self.mag_1_proc_x:>+8.6f};{self.mag_1_proc_y:>+8.6f};{self.mag_1_proc_z:>+8.6f};" \
               f"{self.mag_1_norm:>+8.6f};" \
               f"{self.mag_1_proc_time:>6.5f};" \
               f"{self.mag_2_proc_x:>+8.3f};{self.mag_2_proc_y:>+8.3f};{self.mag_2_proc_z:>+8.3f};" \
               f"{self.mag_2_norm:>+8.3f};" \
               f"{self.mag_2_proc_time:>6.5f};\n"

    @staticmethod
    def csv_header():
        return "gyro_1_proc_x;gyro_1_proc_y;gyro_1_proc_z;gyro_1_proc_time;" \
               "gyro_2_proc_x;gyro_2_proc_y;gyro_2_proc_z;gyro_2_proc_time;" \
               "accel_1_proc_x;accel_1_proc_y;accel_1_proc_z;accel_1_proc_time;" \
               "mag_1_proc_x;mag_1_proc_y;mag_1_proc_z;mag_1_norm;mag_1_proc_time;" \
               "mag_2_proc_x;mag_2_proc_y;mag_2_proc_z;mag_1_norm;mag_2_proc_time;\n"


@dataclass
class ShearWaterEulerPacket:
    roll: float
    pitch: float
    yaw: float
    roll_rate: float
    pitch_rate: float
    yaw_rate: float
    time_stamp: float

    def __repr__(self):
        return f"EulerPacket("\
               f"roll={self.roll:>+8.3f}; pitch={self.pitch:>+8.3f}; yaw={self.yaw:>+8.3f}; "\
               f"roll_rate={self.roll_rate:>+8.3f}; pitch_rate={self.pitch_rate:>+8.3f}; yaw_rate={self.yaw_rate:>+8.3f}; " \
               f"time_stamp={self.time_stamp:>6.3f})"

    def to_csv(self):
        return f"{self.roll:>+8.3f};{self.pitch:>+8.3f};{self.yaw:>+8.3f};" \
               f"{self.roll_rate:>+8.3f};{self.pitch_rate:>+8.3f};{self.yaw_rate:>+8.3f};" \
               f"{self.time_stamp:>6.5f};\n"

    @staticmethod
    def csv_header():
        return "roll;pitch;yaw;roll_rate;pitch_rate;yaw_rate;time_stamp;\n"


@dataclass
class ShearWaterHealthPacket:
    health: int

    def __repr__(self):
        return f"HealthPacket("\
               f"raw_value=0x{self.health:04X}; " \
               f"OVF={bool((self.health >> 8) & 0x01)}, " \
               f"ACC1_N={bool((self.health >> 7) & 0x01)}, " \
               f"MAG1_N={bool((self.health >> 6) & 0x01)}, " \
               f"MAG2_N={bool((self.health >> 5) & 0x01)}, " \
               f"ACCEL1={bool((self.health >> 4) & 0x01)}, "\
               f"GYRO1={bool((self.health >> 3) & 0x01)}, " \
               f"GYRO2={bool((self.health >> 2) & 0x01)}, " \
               f"MAG1={bool((self.health >> 1) & 0x01)}, " \
               f"MAG2={bool((self.health >> 0) & 0x01)})"


@dataclass
class ShearWaterRawAccel1Packet:
    accel_1_raw_x: int
    accel_1_raw_y: int
    accel_1_raw_z: int
    accel_1_raw_time: float


@dataclass
class ShearWaterRawGyro1Packet:
    gyro_1_raw_x: int
    gyro_1_raw_y: int
    gyro_1_raw_z: int
    gyro_1_raw_time: float


@dataclass
class ShearWaterRawGyro2Packet:
    gyro_2_raw_x: int
    gyro_2_raw_y: int
    gyro_2_raw_z: int
    gyro_2_raw_time: float


@dataclass
class ShearWaterRawMag1Packet:
    mag_1_raw_x: int
    mag_1_raw_y: int
    mag_1_raw_z: int
    mag_1_raw_time: float


@dataclass
class ShearWaterRawMag2Packet:
    mag_2_raw_x: int
    mag_2_raw_y: int
    mag_2_raw_z: int
    mag_2_raw_time: float


@dataclass
class ShearWaterTemperaturePacket:
    temperature: float
    temperature_time: float


@dataclass
class ShearWaterProcAccel1Packet:
    accel_1_proc_x: float
    accel_1_proc_y: float
    accel_1_proc_z: float
    accel_1_proc_time: float


@dataclass
class ShearWaterProcGyro1Packet:
    gyro_1_proc_x: float
    gyro_1_proc_y: float
    gyro_1_proc_z: float
    gyro_1_proc_time: float


@dataclass
class ShearWaterProcGyro2Packet:
    gyro_2_proc_x: float
    gyro_2_proc_y: float
    gyro_2_proc_z: float
    gyro_2_proc_time: float


@dataclass
class ShearWaterProcMag1Packet:
    mag_1_proc_x: float
    mag_1_proc_y: float
    mag_1_proc_z: float
    mag_1_proc_norm: float
    mag_1_proc_time: float


@dataclass
class ShearWaterProcMag2Packet:
    mag_2_proc_x: float
    mag_2_proc_y: float
    mag_2_proc_z: float
    mag_2_proc_norm: float
    mag_2_proc_time: float


@dataclass
class ShearWaterQuaternionPacket:
    q_w: float
    q_x: float
    q_y: float
    q_z: float
    q_time: float

    def __repr__(self):
        return f"QuaternionPacket("\
               f"q_w={self.q_w:>+3.5f}; q_x={self.q_x:>+3.5f}; q_y={self.q_y:>+3.5f}; q_z={self.q_z:>+3.5f};"\
               f" time_stamp={self.q_time:>6.5f})"

    def to_csv(self):
        return f"{self.q_w:>+3.5f};{self.q_x:>+3.5f};{self.q_y:>+3.5f};{self.q_z:>+3.5f};" \
               f"{self.q_time:>6.5f};\n"

    @staticmethod
    def csv_header():
        return "q_w;q_x;q_y;q_z;q_time;\n"


@dataclass
class ShearWaterEulerPosePacket:
    roll: float
    pitch: float
    yaw: float
    roll_rate: float
    pitch_rate: float
    yaw_rate: float
    euler_time: float
    position_north: float
    position_east: float
    position_up: float
    position_time: float


@dataclass
class ShearWaterPosePacket:
    position_north: float
    position_east: float
    position_up: float
    position_time: float


@dataclass
class ShearWaterVelocityPacket:
    velocity_north: float
    velocity_east: float
    velocity_up: float
    velocity_time: float


@dataclass
class ShearWaterGyro1BiasPacket:
    gyro_1_bias_x: float
    gyro_1_bias_y: float
    gyro_1_bias_z: float


@dataclass
class ShearWaterGyro2BiasPacket:
    gyro_2_bias_x: float
    gyro_2_bias_y: float
    gyro_2_bias_z: float
